/* fpmap.c - frequency to place map */

#include <stdio.h>

#define NPTS 5

double  cf();
double  atof();
double  pow();

static char *ifn[NPTS] = {
    "dt3a52.plt",
    "dt3a42.plt",
    "dt3a32.plt",
    "dt3a22.plt",
    "dt3a12.plt",
};
static float place[NPTS] = {90, 80, 60, 40, 20};

int
main()
{
    fpmap_("dat4a0.plt", 0.165, 0.021, 0.99, "man");
    fpmap_("dat4a1.plt", 0.456, 0.021, 0.80, "cat");
    fpmap2("dat4a2.plt");

    return (0);
}

fpmap_(ofn, a, b, c, spnm)
char *ofn, *spnm;
double a, b, c;
{
    int     i;
    float   f[101], p[101];
    FILE   *ofp;

    for (i = 0; i < 101; i++) {
	p[i] = i;
	f[i] = a * (pow(10.0, b * (100 - i)) - c);
    }
    if ((ofp = fopen(ofn, "w")) == NULL) {
	fprintf(stderr, "can't open %s\n", ofn);
	exit(0);
    }
	fprintf(ofp, 
	    ";%s - Greenwood frequency-to-place map for %s\n"
	    ";\n"
	    ";	a=%.3f,  b=%.3f, c=%.3f\n"
	    ";\n"
	    ";	x=frequency (kHz)\n"
	    ";	y=percent distance from base\n"
	    ";\n", ofn, spnm, a, b, c);
    for (i = 0; i < 101; i++)
	fprintf(ofp, "%.3f %.3f\n", f[i], p[i]);
    fclose(ofp);
}

fpmap2(ofn)
char *ofn;
{
    int     i;
    float   f[NPTS], p[NPTS];
    FILE   *ofp;

    for (i = 0; i < NPTS; i++) {
	f[i] = cf(ifn[i]);
	p[i] = place[i];
    }
    if ((ofp = fopen(ofn, "w")) == NULL) {
	fprintf(stderr, "can't open %s\n", ofn);
	exit(0);
    }
	fprintf(ofp, 
	    ";%s - model frequency-to-place map\n"
	    ";\n"
	    ";	x=frequency (kHz)\n"
	    ";	y=percent distance from base\n"
	    ";\n", ofn);
    for (i = 0; i < NPTS; i++)
	fprintf(ofp, "%.3f %.3f\n", f[i], p[i]);
    fclose(ofp);
}

/* cf - characteristic frequency */

double
cf(fn)
char   *fn;
{
    char    line[80];
    int     i, j;
    float   f, p, fmin, pmin;
    FILE   *ifp;

    fmin = 0;
    pmin = 1e18;
    if ((ifp = fopen(fn, "r")) == NULL) {
	fprintf(stderr, "can't open %s\n", fn);
	return (0);
    }
    for (i = 0; fgets(line, 80, ifp) != NULL; i++) {
	if ((f = atof(line)) > 0) {
	    for (j = 0; line[j] == ' '; j++)
		continue;
	    for (; line[j] > ' '; j++)
		continue;
	    for (; line[j] == ' '; j++)
		continue;
	    p = atof(line + j);
	    if (pmin > p) {
		pmin = p;
		fmin = f;
	    }
	}
    }
    fclose(ifp);
    return (fmin);
}
