/* getped - get tympanic pressure data from file */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#define MAIN
#include "tdmdef.h"
#include "cparam.h"
#include "cstate.h"

#define MXNP 2050

void swab();

static int filopn = 0;
static FILE *dfp = NULL;
static char msg[80];
static int npnt, nstp;
static int swap_bytes = 0;
static float dtms = 0.0;
static float dat[MXNP];

double
spl(double p)
{
    static float pref = 2e-4;	/* dyn/cm**2 */

    return (20.0 * log10(p / pref));
}

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int     n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

static void
error(s)
char   *s;
{
    fprintf(stderr, "ERROR:  %s\n", s);
    exit(1);
}

static void
opndat(fn)
char   *fn;
{
    if (filopn)
	fclose(dfp);
    if ((dfp = fopen(fn, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn);
	error(msg);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	error("reading header\n");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *) &pa_.magic, 1);
	swab((void *) &pa_.hdrsiz, (void *) &pa_.hdrsiz, 6 * sizeof(short));
	swabw((long *) &pa_.dt, 14);
	swab((void *) &pa_.m, (void *) &pa_.m, 21 * sizeof(short));
	swabw((long *) &pa_.xl, 164);
	swap_bytes++;
    }
    if (pa_.ireor == -1) {	/* read state */
	fseek(dfp, pa_.hdrsiz, 0);
	if (fread((char *) &st_, sizeof(st_), 1, dfp) < 1)
	    error("error reading state");
	if (swap_bytes)
	    swabw((long *) (char *) &st_, sizeof(st_) / sizeof(float));
    } else {
	if (pa_.ireor == 1) {
	    nstp = 16;
	} else if (pa_.ireor == 2) {
	    nstp = 1;
	} else {
	    error("undefined file type");
	}
	if (pa_.ifrmt != 8 && pa_.ifrmt != 7) {
	    error("undefined file format");
	}
	npnt = pa_.ntsf;
	dtms = 1000 * pa_.ntsw * pa_.dt;
	printf("opened file: %s, npnt=%d, nstp=%d\n", fn, npnt, nstp);
	if (npnt > MXNP) {
	    npnt = MXNP;
	    printf("WARNING: reducing npnt to %d\n", npnt);
	}
    }
    filopn = 1;
}

static void
getdat(int istp, float *ff)
{
    long    hsiz, nbyt;

    if (pa_.ireor == -1)
	return;
    if (!filopn)
	error("getdat requires file to be assigned");
    hsiz = pa_.hdrsiz;
    nbyt = npnt * sizeof(float);
    fseek(dfp, hsiz + nbyt * (istp - 1), 0);
    if (fread((char *) ff, npnt, sizeof(float), dfp) < 1)
	        error("reading data");
    if (swap_bytes)
	swabw((long *) ff, npnt);
}

static void
maxrms(float *p, int n, float *max, float *rms)
{
    float   pmax, psum, pssq, pmsq;
    int     i;

    pmax = 1e-38;
    psum = 0;
    pssq = 0;
    for (i = 0; i < n; i++) {
	if (pmax < p[i])
	    pmax = p[i];
	psum += p[i];
	pssq += p[i] * p[i];
    }
    pmsq = psum * psum / n;
    *rms = sqrt((pssq - pmsq) / n);
    *max = pmax;
}

static void
pedspl()
{
    float   pmax, psum, pssq, prms, pmsq, np;

    if (pa_.ireor == -1) {
	pmax = st_.pedmax;
	psum = st_.pedsum;
	pssq = st_.pedssq;
	if ((np = st_.pednum) > 0) {
	    pmsq = psum * psum / np;
	    prms = sqrt((pssq - pmsq) / np);
	}
    } else {
	getdat(5, dat);
	maxrms(dat, npnt, &pmax, &prms);
    }
    printf("peak pressure at eardrum = %.3e dyn/cm^2  ", pmax);
    printf("= %4.1f dB pSPL\n", spl(pmax));
    printf(" rms pressure at eardrum = %.3e dyn/cm^2  ", prms);
    printf("= %4.1f dB SPL\n", spl(prms));
}

static void
usage()
{
    fprintf(stderr, "usage: getped tsr_file\n");
    exit(1);
}

int
main(ac, av)
char  **av;
{
    if (ac < 2)
	usage();
    opndat(av[1]);
    pedspl();

    return (0);
}
