/* fttsd.c */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "tdmdef.h"
#define MAIN
#include "cparam.h"

#define NDAT 12

void swab();

static char fn1[40] = "tdm.tsd";
static char fn2[40] = "tdm.ftd";
static char fn3[40] = "tdm.cal";
static int dbgflg = 1;
static int swap_bytes = 0;
static FILE *fp1, *fp2, *fp3;

void error();
void newext();
void rfft();

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

void
wrmap(float *b, int nrows, int ncols, double dx, double df)
{
    double db;
    int i, j, nf, im;
    float *c, mg, mx;
    FILE *fp;

    nf = ncols / 2;
    fp = fopen("tdm.map", "wt");
    fprintf(fp, "; tdm.map - frequency-place map, L=%.2f\n", pa_.xl - pa_.xhe);
    fprintf(fp, ";\n");
    fprintf(fp, ";    i      x/L       f       d (dB)\n");
    for (j = 0; j < nrows; j++) {
        c = b + j * ncols;
	mx = 0;
	im = 0;
        for (i = 1; i < nf; i++) {
            mg = c[2 * i];
	    if (mx < mg) {
		mx = mg;
		im = i;
	    }
	}
	db = (mx > 1e-30) ? 20 * log10(mx) + 200 : -600;
	fprintf(fp, "%6d %9.3f %9.1f %9.2f\n", j, j * dx, im * df, db);
    }
    fclose(fp);
}

int
main(int ac, char **av)
{
    char    msg[80];
    double  tp, sc, ped, pst, dat[NDAT], vep, sv0, sv1, sv2, wnr, dx, df;
    int     nrows, ncols, npts, i, j, ir, ii, nf;
    int     fc[32], nc = 0, mapflg = 0;
    long    hsiz;
    float  *a, *b, *c, cmg, cph;
    static char *datlab[NDAT] = {
    	";  f2 ", "   f1 ", "   L2 ", "   L1 ", "   fdp", "   Ldp",
    	"   Pst", "   <dp", "   D2 ", "   D1 ", "   Ddp", "   WNR"
    };
    static double rad2deg = 180 / 3.1415927;
    static double dsp_ref = 1e-10;
    static double spl_ref = 0.0002 * 1.4142;


    if (ac > 1) {
	while (av[1][0] == '-') {
	    if (av[1][1] == 'm') {
		mapflg = 1;
	    }
	    ac--;
	    av++;
	}
	strcpy(fn1, av[1]);
	strcpy(fn2, (ac > 2) ? av[2] : av[1]);
    }
    newext(fn1, ".tsd");
    newext(fn2, ".ftd");

    if ((fp1 = fopen(fn1, "rb")) == NULL) {
	sprintf(msg, "can't open %s", fn1);
	error(msg);
    }
    fprintf(stderr, "opened %s\n", fn1);
    if (fread((char *) &pa_, sizeof(pa_), 1, fp1) < 1) {
	error("reading header");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
    if (dbgflg) {
	printf(" ireor,ifrmt,jfrmt    = %4d%4d%4d\n",
	    pa_.ireor, pa_.ifrmt, pa_.jfrmt);
	printf(" itsc,icyc,itsw,itsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);
	printf(" ntsc,ncyc,ntsw,ntsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    }
    if (pa_.ireor != 0) {
	sprintf(msg, "wrong type of data file: ireor=%d", pa_.ireor);
	error(msg);
    }
    ncols = pa_.ntsf;
    switch (pa_.ifrmt) {
    case 1:
	nrows = pa_.n;
	break;
    case 2:
	nrows = pa_.n * 2;
	break;
    case 3:
    case 7:
    case 8:
	nrows = 16;
	break;
    case 5:
    case 6:
	nrows = 256;
	break;
    default:
	sprintf(msg, "format not implemented: ifrmt=%d\n", pa_.ifrmt);
	error(msg);
    }

    npts = nrows * ncols;
    printf("nrows=%d  ncols=%d  npts=%d\n", nrows, ncols, npts);

    a = (float *) malloc(nrows * sizeof(float));
    b = (float *) malloc(nrows * ncols * sizeof(float));
    if (a == NULL || b == NULL) {
	error("allocating memory");
    }
    hsiz = pa_.hdrsiz;
    fseek(fp1, hsiz, 0);
    for (i = 0; i < ncols; i++) {
	if (fread((char *) a, sizeof(float), nrows, fp1) < nrows) {
	    /* error("reading data") */;
	}
	if (swap_bytes)
	    swabw((long *) a, nrows);
	for (j = 0; j < nrows; j++)
	    b[i + j * ncols] = a[j];
    }
    fclose(fp1);

    nf = ncols / 2;
    sc = (2.0 / ncols);
    for (j = 0; j < nrows; j++) {
        c = b + j * ncols;
        rfft(c, ncols);
	c[0] /= ncols;
	c[1] /= ncols;
        for (i = 1; i < nf; i++) {
            ir = 2 * i;
            ii = 2 * i + 1;
            cmg = hypot(c[ii],c[ir]) * sc;
            cph = atan2(c[ii],c[ir]);
            c[ir] = cmg;
            c[ii] = cph;
	}
    }
    wnr = b[0 + 2 * ncols];
    if (mapflg) {
	dx = (pa_.xl / (nrows - 1)) / (pa_.xl - pa_.xhe);
	df = 1 / (pa_.ntsf * pa_.ntsw * pa_.dt);
	wrmap(b, nrows, ncols, dx, df);
    }
    pa_.ireor = 2;		/* identify output file as FT data */
    if ((fp2 = fopen(fn2, "wb")) == NULL) {
	sprintf(msg, "can't open %s", fn2);
	error(msg);
    }
    if (fwrite((char *) &pa_, sizeof(pa_), 1, fp2) < 1) {
	error("writing header");
    }
    fseek(fp2, hsiz, 0);
    if (pa_.f1 <= 0) {
        for (i = 0; i < ncols; i++) {
            for (j = 0; j < nrows; j++)
                a[j] = b[i + j * ncols];
            if (fwrite((char *) a, sizeof(float), nrows, fp2) < nrows)
                error("writing data");
        }
        if (pa_.ifrmt == 8) {
            fp3 = fopen(fn3, "w");
            c = b + 4 * ncols;
            tp = pa_.ntsf * pa_.ntsw * pa_.dt;
            for (i = 1; i < nf; i++) {
                ir = 2 * i;
                vep = (b[ir] > 0) ? 20 * log10(b[ir] / pa_.stgain) : -99;
                ped = (c[ir] > 0) ? 20 * log10(c[ir] / spl_ref) : -99;
                fprintf(fp3, "%6.0f %7.2f\n", i / tp, vep - ped);
            }
            fclose(fp3);
        }
    } else {
        tp = pa_.ntsf * pa_.ntsw * pa_.dt;
        nc = 0;
        fc[nc] = pa_.f1 * tp + 0.5;
        if (fc[nc] > 0) nc++;
        fc[nc] = pa_.f2 * tp + 0.5;
        if (fc[nc] > 0) nc++;
        fc[nc] = 2 * fc[0] - fc[1];
        if (fc[nc] > 0) nc++;
        fc[nc] = fc[1] - fc[0];
        if (fc[nc] > 0) nc++;
        for (i = 0; i < nc; i++) {
            ir = 2 * fc[i];
            ii = ir + 1;
            for (j = 0; j < nrows; j++)
                a[j] = b[ir + j * ncols];
            if (fwrite((char *) a, sizeof(float), nrows, fp2) < nrows)
                error("writing data");
            if (pa_.ifrmt == 8) {
                ped = (a[4] > 0) ? 20 * log10(a[4] / spl_ref) : -99;
                pst = (a[8] > 0) ? 20 * log10(a[8] / spl_ref) : -99;
                sv0 = (a[12] > 0) ? 20 * log10(a[12] / dsp_ref) : -99;
                sv1 = (a[13] > 0) ? 20 * log10(a[13] / dsp_ref) : -99;
                sv2 = (a[14] > 0) ? 20 * log10(a[14] / dsp_ref) : -99;
            }
            for (j = 0; j < nrows; j++)
                a[j] = b[ii + j * ncols];
            if (fwrite((char *) a, sizeof(float), nrows, fp2) < nrows)
                error("writing data");
            if (pa_.ifrmt != 8) {
                printf("fc[%d]=%3d (%4.0f)\n", i, fc[i], fc[i] / tp);
            } else {
                if (i == 0) {
                    dat[1] = fc[0] / tp;
                    dat[3] = ped;
                    dat[9] = sv1;
                } else if (i == 1) {
                    dat[0] = fc[1] / tp;
                    dat[2] = ped;
                    dat[8] = sv0;
                } else if (i == 2) {
                    dat[4] = fc[2] / tp;
                    dat[5] = ped;
                    dat[6] = pst;
                    dat[7] = a[4] * rad2deg;
                    dat[10] = sv2;
                    dat[11] = (wnr > 0) ? 20 * log10(wnr) : -99;
                }
            }
        }
        if (pa_.ifrmt == 8) {
            for (i = 0; i < NDAT; i++)
                printf("%s ", datlab[i]);
            printf("\n");
            printf("%6.0f %6.0f %6.1f %6.1f %6.0f %6.1f ", 
                dat[0], dat[1], dat[2], dat[3], dat[4], dat[5]);
            for (i = 6; i < NDAT; i++)
                printf("%6.1f ", dat[i]);
            printf("\n");
        }
    }
    fclose(fp2);

    free(a);
    free(b);
    return(0);
}

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}

void
error(s)
char   *s;
{
    fprintf(stderr, "ERROR: %s\n", s);
    exit(1);
}
