/* mkfig3 - make plt files for ped and wnr */

#include <stdio.h>
#include <ctype.h>
#define MAIN
#include "tdmdef.h"
#include "cparam.h"

#define MXNP 2050

char   *strstr();
double  log10();
int     ipeak();

static float tsep = 0.05;
static int dtset = 1;
static int npk = 0;
static double eps = 1e-38;

static int filopn = 0;
static int verbose = 0;
static FILE *dfp = NULL;
static char msg[80];
static int npnt, nstp;
static int swap_bytes = 0;
static float dtms = 0.0;
static float aa[MXNP];
static float wnrpk[4];
static float wnrkn[4];
static float xpk[4];
static float xkn[4];
static int sym[4];

int
main(ac, av)
char  **av;
{
    char    ifn[80], *lc;
    int     i;

    if (ac < 2)
	usage();
    strcpy(ifn, av[1]);
    opndat(ifn);

    if ((lc = strstr(ifn, "a.tsr")) == NULL)
	error("expected '...a.tsr' in filename");
    i = lc - ifn;

    pltxmx('d', i, ifn, "fig3a1.dat", 5);
    pltxmx('c', i, ifn, "fig3a2.dat", 4);
    pltxmx('b', i, ifn, "fig3a3.dat", 1);
    pltxmx('a', i, ifn, "fig3a4.dat", 2);
    makemark("fig3a5.dat", wnrpk, xpk, npk);
    makemark("fig3a6.dat", wnrkn, xkn, npk);

    return (0);
}

pltxmx(c, i, ifn, ofn, sn)
char   *ifn, *ofn;
{
    int ipk, ikn;

    ifn[i] = c;
    opndat(ifn);
    getdat(5);
    cmpspl(aa, npnt);
    getdat(3);
    ipk = ipeak(aa, npnt);
    vderiv(aa, aa, npnt);
    ikn = ipeak(aa, npnt);
    getdat(2);
    maketime(ofn, aa, npnt);
    wnrpk[npk] = ipk * dtms;
    xpk[npk] = aa[ipk];
    wnrkn[npk] = ikn * dtms;
    xkn[npk] = aa[ikn];
    sym[npk] = sn;
    npk++;
}

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

opndat(fn)
char   *fn;
{
    if (filopn)
	fclose(dfp);
    if ((dfp = fopen(fn, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn);
	error(msg);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	error("reading header\n");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
    if (pa_.ifrmt != 8 && pa_.ifrmt != 7) {
	error("undefined file format");
    } else if (pa_.ireor == 1) {
	nstp = 16;
    } else if (pa_.ireor == 2) {
	nstp = 1;
    } else {
	error("undefined file type");
    }
    npnt = pa_.ntsf;
    dtms = 1000 * pa_.ntsw * pa_.dt;
    printf("opened file: %s, npnt=%d, nstp=%d\n", fn, npnt, nstp);
    if (npnt > MXNP) {
	npnt = MXNP;
	printf("WARNING: reducing npnt to %d\n", npnt);
    }
    filopn = 1;
}

getdat(istp)
{
    long    hsiz, nbyt;
    float  *ff;

    if (!filopn)
	error("getped requires file to be assigned");
    ff = aa;
    hsiz = pa_.hdrsiz;
    nbyt = npnt * sizeof(float);
    fseek(dfp, hsiz + nbyt * (istp - 1), 0);
    if (fread((char *) ff, nbyt, 1, dfp) < 1)
	error("reading data");
}

scale()
{
    int     i;
    float  *ff, mxmg;

    ff = aa;
    mxmg = eps;
    for (i = 0; i < npnt; i++) {
	if (mxmg < ff[i])
	    mxmg = ff[i];
	if (mxmg < -ff[i])
	    mxmg = -ff[i];
    }
    for (i = 0; i < npnt; i++) {
	ff[i] /= mxmg;
    }
    if (verbose)
	printf("SC: mxmg=%.3g\n", mxmg);
}

maketime(ofn, ff, np)		/* write data vs time */
char   *ofn;
float  *ff;
{
    int     i;
    float   time, dtime, lastt;
    FILE   *ofp;

    if ((ofp = fopen(ofn, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", ofn);
	error(msg);
    }
    if (verbose)
	printf("writing time data to %s\n", ofn);
    lastt = 0;
    dtime = dtms;
    for (i = 0; i < np; i++) {
	time = i * dtime;
	if (dtset) {
	    if ((time - lastt) < tsep)
		continue;
	}
	fprintf(ofp, "%9.4f %9.4f\n", time, ff[i]);
	lastt = time;
    }
    fclose(ofp);
}

makemark(ofn, ff, gg, np)		/* write data vs time */
char   *ofn;
float  *ff, *gg;
{
    int     i;
    FILE   *ofp;

    if ((ofp = fopen(ofn, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", ofn);
	error(msg);
    }
    for (i = 0; i < np; i++) {
	fprintf(ofp, "symbol=%d\n", sym[i]);
	fprintf(ofp, "%9.4f %9.4f\n", ff[i], gg[i]);
	fprintf(ofp, "plot\n");
    }
    fclose(ofp);
}

cmpspl(p, n)
float  *p;
{
    float   pmax, prat, spl;
    int     i;
    static float pref = 2.828e-4;	/* dyn/cm**2 */

    pmax = 1e-38;
    for (i = 0; i < n; i++)
	if (pmax < p[i])
	    pmax = p[i];
    prat = pmax / pref;
    spl = 20.0 * log10(prat);
    printf("peak pressure at eardrum = %.3e dyn/cm^2  ", pmax);
    printf("= %.1f peSPL\n", spl);
}

ipeak(p, n)
float  *p;
{
    float   pmax;
    int     i, imax;

    pmax = p[0];
    imax = 0;
    for (i = 1; i < n; i++)
	if (pmax < p[i]) {
	    pmax = p[i];
	    imax = i;
	}
    return (imax);
}

char   *
strstr(s1, s2)
char   *s1, *s2;
{
    int     i;

    while (*s1) {
	if (*s1 == *s2) {
	    for (i = 0; s1[i] && s2[i] && s1[i] == s2[i]; i++)
		continue;
	    if (s2[i] == '\0')
		return (s1);
	}
	s1++;
    }
    return (NULL);
}

vderiv(a, b, n)
float *a, *b;
{
    int i;
    float atmp, btmp;

    atmp = a[0];
    for (i = 1; i < n; i++) {
	btmp = a[i];
	b[i] = btmp - atmp;
	atmp = btmp;
    }
}

usage()
{
    fprintf(stderr, "usage: fig5 tsr_file \n");
    exit(1);
}

error(s)
char   *s;
{
    fprintf(stderr, "ERROR:  %s\n", s);
    exit(1);
}
