/* outlib - "libout" graphics functions for GRX & XW */

#include <stdlib.h>
#include <stdio.h>
#include <string.h> 
#include <unistd.h>

#ifdef MSDOS
#include <pc.h>
#endif /* MSDOS */

#ifdef GRX
#include <grx20.h>
#include <grdriver.h>
#include <grfontdv.h>
static int fgc = 0, bgc = 0;
#endif /* GRX */

#ifdef XW
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#endif /* XW */

static double xo = 0, yo = 0;
static double xg = 0, yg = 0;
static int xp = 0, yp = 0;
static int pgb = 1, pgp = 1;
static int xpix = 0, ypix = 0;
static int grfmod = 0;

#ifdef XW
static Colormap cmap;
static Display *mdsp;
static Window  mwin;
static GC      mgc;
static GC      egc;
static unsigned long wp;
static unsigned long bp;
#endif /* XW */

int
defdev()
{
    int dd = 0;
#ifdef XW
    if (getenv("DISPLAY"))
	dd = 2;
#endif /* XW */
#ifdef GRX
#ifdef MSDOS
    dd = 1;
#else /* MSDOS */
    if (!dd && access("/dev/console", W_OK) == 0)
	dd = 1;
#endif /* MSDOS */
#endif /* GRX */
    return (dd);
}

int
opnout(int mode)
{
#ifdef GRX
    if (mode == 1) {
	grfmod = 1;
	GrSetMode(GR_default_graphics); 
	fgc = GrWhite();
	bgc = GrBlack();
	xpix = GrSizeX();
	ypix = GrSizeY();
    }
#endif /* GRX */
#ifdef XW
    if (mode == 2) {
	int     border_width = 2;	/* border width of the window */
	int     dpth;
	int     scrn;
	int     x = 8;		/* x top left corner of window */
	int     y = 8;		/* y top left corner of window */
	unsigned int height = 480;	/* height of the window */
	unsigned int width = 640;	/* width of the window */
	unsigned long wmsk;
	XGCValues gcv;
	XSetWindowAttributes swa;
	XSizeHints theSizeHints;
	XWindowAttributes wa; 

	mdsp = XOpenDisplay("");
	if (mdsp == NULL) {
	    return(0);
	}
	scrn = DefaultScreen(mdsp); 
	dpth = DefaultDepth(mdsp, scrn); 
	wp = WhitePixel(mdsp, scrn);
	bp = BlackPixel(mdsp, scrn);
	cmap = DefaultColormap(mdsp, scrn); 
	x = (XDisplayWidth(mdsp, scrn) - width) / 2;
	y = 8;
	swa.border_pixel = BlackPixel(mdsp, scrn); 
	swa.background_pixel = WhitePixel(mdsp, scrn); 
	swa.override_redirect = True;
	swa.backing_store = WhenMapped;
	wmsk = CWBackPixel | CWBorderPixel | CWOverrideRedirect
	    | CWBackingStore; 
	mwin = XCreateWindow(mdsp,
	    RootWindow(mdsp, scrn),
	    x, y, width, height, border_width,
	    dpth, InputOutput, CopyFromParent,
	    wmsk, &swa); 
	theSizeHints.flags = PPosition | PSize;	/* set mask for the hints */
	theSizeHints.x = x;		/* x position */
	theSizeHints.y = y;		/* y position */
	theSizeHints.width = width;	/* width of the window */
	theSizeHints.height = height;	/* height of the window */ 
	XSetNormalHints(mdsp, mwin, &theSizeHints); 
	mgc = XCreateGC(mdsp, mwin, (unsigned long) 0, &gcv); 
	egc = XCreateGC(mdsp, mwin, (unsigned long) 0, &gcv); 
	if (mgc == 0 || egc == 0) { /* error... cannot create gc */
	    XDestroyWindow(mdsp, scrn);
	    exit(0);
	} else { /* set forground and background defaults */
	    XSetForeground(mdsp, mgc, bp);
	    XSetBackground(mdsp, mgc, wp);
	    XSetForeground(mdsp, egc, wp);
	    XSetBackground(mdsp, egc, wp);
	} 
	XMapRaised(mdsp, mwin);
	XGetWindowAttributes(mdsp, mwin, &wa); 
	//*xs = wa.width;
	//*ys = wa.height;
	xpix = wa.width;
	ypix = wa.height;
	grfmod = 2;
    } 
#endif /* XW */
    xo = 0;
    yo = ypix - 1;
    xg = xpix / 10.5;
    yg = -ypix / 8.0;
    return (grfmod);
}

void
setchh(int chh)
{
}

void
setfais(int fais)
{
}

void
setfasi(int fasi)
{
}

void
setpgb(int b)
{
    pgb = b;
}

void
setpgp(int p)
{
    pgp = p;
}

void
text(double x, double y, char *s)
{
    int ix, iy;
    
    ix = (int) (x * xg + xo);
    iy = (int) (y * yg + yo);
#ifdef GRX
    if (grfmod == 1)
	GrTextXY(ix, iy, s, fgc, bgc);
#endif /* GRX */
#ifdef XW
    if (grfmod == 2)
	XDrawString(mdsp, mwin, mgc, ix, iy, s, strlen(s));
#endif /* XW */
}

void
move(double x, double y)
{
    xp = (int) (x * xg + xo);
    yp = (int) (y * yg + yo);
}

void
draw(double x, double y)
{
    int ix, iy;
    
    ix = (int) (x * xg + xo);
    iy = (int) (y * yg + yo);
#ifdef GRX
    if (grfmod == 1)
	GrLine(xp, yp, ix, iy, fgc); 
#endif /* GRX */
#ifdef XW
    if (grfmod == 2)
	XDrawLine(mdsp, mwin, mgc, xp, yp, ix, iy);
#endif /* XW */
    xp = ix;
    yp = iy;
}

void
page()
{
#ifdef XW
    if (grfmod == 2)
	XFlush(mdsp);
#endif /* XW */
    if (pgp)
        getchar();
#ifdef GRX
    if (grfmod == 1)
	GrClearScreen(bgc);
#endif /* GRX */
#ifdef XW
    if (grfmod == 2) {
	XFillRectangle(mdsp, mwin, egc, 0, 0, xpix, ypix);
    }
#endif /* XW */
}

void
clsout()
{
    if (pgp)
        getchar();
#ifdef GRX
    if (grfmod == 1)
	GrSetMode(GR_default_text);
#endif /* GRX */
#ifdef XW
    if (grfmod == 2)
	;
#endif /* XW */
}

