/* pwrflo.c */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "tdmdef.h"
#define MAIN
#include "cparam.h"

void rfft();
void swab();

static char fn1[40] = "tdm.tsd";
static char fn2[40] = "pwrflo.dat";
static char fn3[40] = "bmvel.dat";
static char fn4[40] = "bmdis.dat";
static char fn5[40] = "press.dat";
static char fn6[40] = "bmvph.dat";
static int dbgflg = 1;
static int swap_bytes = 0;
static FILE *fp1, *fp2, *fp3, *fp4, *fp5, *fp6;

/* swabw - swap bytes (in place) for n "words" */
void
swabw(p, n)
long   *p;
int n;
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}

void
error(s)
char   *s;
{
    fprintf(stderr, "ERROR: %s\n", s);
    exit(1);
}

int
main(ac, av)
char  **av;
{
    char    msg[80];
    double  f, tp, dx, tpi, w, sc;
    double  pv, ps, pm, vs, vm, vp;
    double  pso[32];
    int     nrows, ncols, npts, nofst;
    int     i, j, ir, ii, j1, j2;
    int     fc[32], nc = 0;
    long    hsiz;
    float  *a, *b, *p, *v;


    while (ac > 1 && av[1][0] == '-') {
        if (av[1][1] == 'e') {
            newext(fn2, av[1] + 2);
            newext(fn3, av[1] + 2);
            newext(fn4, av[1] + 2);
            newext(fn5, av[1] + 2);
            newext(fn6, av[1] + 2);
            printf("output files: %s %s %s %s %s\n",
		fn2, fn3, fn4, fn5, fn6);
        } else {
	    printf("usage:  pwrflo [-e.X] [tsd_file [f1 f2 ...]]\n");
	    exit(1);
        }
        ac--;
        av++;
    }
    if (ac > 1) {
        strcpy(fn1, av[1]);
    }

    if ((fp1 = fopen(fn1, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", fn1);
	error(msg);
    }
    fprintf(stderr, "opened %s\n", fn1);
    if (fread((char *) &pa_, sizeof(pa_), 1, fp1) < 1) {
	error("reading header\n");
    }
    if (pa_.ireor > 255) {	/* check for byte swap */
	swabw((long *)&pa_.magic, 1);
	swab((void*)&pa_.hdrsiz, (void *)&pa_.hdrsiz, 6 * sizeof(short) );
	swabw((long *)&pa_.dt, 14);
	swab((void*)&pa_.m, (void*)&pa_.m, 21 * sizeof(short));
	swabw((long *)&pa_.xl, 164);
	swap_bytes++;
    }
    if (dbgflg) {
	printf(" ireor,ifrmt,jfrmt    = %4d%4d%4d\n",
	    pa_.ireor, pa_.ifrmt, pa_.jfrmt);
	printf(" itsc,icyc,itsw,itsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);
	printf(" ntsc,ncyc,ntsw,ntsf  = %8ld%8ld%8ld%8ld\n",
	    pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    }
    if (pa_.ireor != 0) {
	sprintf(msg, "wrong type of data file: ireor=%d\n", pa_.ireor);
	error(msg);
    }
    if (pa_.ifrmt != 2 || pa_.jfrmt != 4) {
	sprintf(msg, "wrong data format: ifrmt=%d, jfrmt=%d\n",
	    pa_.ifrmt, pa_.jfrmt);
	error(msg);
    }

    nrows = pa_.n * 2;
    ncols = pa_.ntsf;
    nofst = pa_.ntsf * pa_.n;
    npts = nrows * ncols;
    printf("nrows=%d  ncols=%d  npts=%d\n", nrows, ncols, npts);
    tp = pa_.ntsf * pa_.ntsw * pa_.dt;
    printf("dt=%.3f ms, df=%.3f Hz\n", tp * 1000, 1 / tp);
    j1 = 0;
    j2 = pa_.n - 1;
    if (ac < 3) {
        fc[0] = pa_.f1 * tp + 0.5;
        if (pa_.f2 == 0) {
            nc = 1;
        } else {
            fc[1] = pa_.f2 * tp + 0.5;
            fc[2] = 2 * fc[0] - fc[1];
            nc = 3;
        }
    } else {
        for (i = 2; i < ac; i++) {
	    f = atof(av[i]);
	    j = f * tp + 0.5;
	    if (j > 1 && j < (pa_.ntsf / 2)) {
	        fc[nc++] = j;
	    }
	}
    }
    for (i = 0; i < nc; i++)
	printf("fc[%d]=%d (%.0f)\n", i, fc[i], fc[i] / tp);

    a = (float *) malloc(nrows * sizeof(float));
    b = (float *) malloc(nrows * ncols * sizeof(float));
    if (a == NULL || b == NULL) {
	error("allocating memory\n");
    }
    hsiz = pa_.hdrsiz;
    fseek(fp1, hsiz, 0);
    for (i = 0; i < ncols; i++) {
	if (fread((char *) a, sizeof(float), nrows, fp1) < nrows) {
	    /* error("reading data\n") */;
	}
	if (swap_bytes)
	    swabw((long *) a, nrows);
	for (j = 0; j < nrows; j++)
	    b[i + j * ncols] = a[j];
    }
    fclose(fp1);

    if ((fp2 = fopen(fn2, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", fn2);
	error(msg);
    }
    if ((fp3 = fopen(fn3, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", fn3);
	error(msg);
    }
    if ((fp4 = fopen(fn4, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", fn4);
	error(msg);
    }
    if ((fp5 = fopen(fn5, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", fn5);
	error(msg);
    }
    if ((fp6 = fopen(fn6, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", fn6);
	error(msg);
    }

    fprintf(fp2, "; %s\n", fn2);
    fprintf(fp2, ";\n");
    fprintf(fp2, "; x (cm)");
    for (i = 0; i < nc; i++)
	fprintf(fp2, "  pf[%6.0f]  ", fc[i] / tp);
    fprintf(fp2, "\n");

    fprintf(fp3, "; %s\n", fn3);
    fprintf(fp3, ";\n");
    fprintf(fp3, "; x (cm)");
    for (i = 0; i < nc; i++)
	fprintf(fp3, "  v1[%6.0f]  ", fc[i] / tp);
    fprintf(fp3, "\n");

    fprintf(fp4, "; %s\n", fn4);
    fprintf(fp4, ";\n");
    fprintf(fp4, "; x (cm)");
    for (i = 0; i < nc; i++)
	fprintf(fp4, "  d1[%6.0f]  ", fc[i] / tp);
    fprintf(fp4, "\n");

    fprintf(fp5, "; %s\n", fn5);
    fprintf(fp5, ";\n");
    fprintf(fp5, "; x (cm)");
    for (i = 0; i < nc; i++)
	fprintf(fp5, "   p[%6.0f]  ", fc[i] / tp);
    fprintf(fp5, "\n");

    fprintf(fp6, "; %s\n", fn6);
    fprintf(fp6, ";\n");
    fprintf(fp6, "; x (cm)");
    for (i = 0; i < nc; i++)
	fprintf(fp6, "  vp[%6.0f]  ", fc[i] / tp);
    fprintf(fp6, "\n");

    dx = pa_.xl / (pa_.n - 1);
    sc = (2.0 / ncols) * (2.0 / ncols);
    tpi = 8 * atan(1.0);
    for (j = j1; j < j2; j++) {
	p = b + j * ncols;
	v = p + nofst;
	rfft(p, ncols);
	rfft(v, ncols);
	fprintf(fp2, "%.2f",  j * dx);
	fprintf(fp3, "%.2f",  j * dx);
	fprintf(fp4, "%.2f",  j * dx);
	fprintf(fp5, "%.2f",  j * dx);
	fprintf(fp6, "%.2f",  j * dx);
	for (i = 0; i < nc; i++) {
	    ir = fc[i] * 2;
	    ii = ir + 1;
	    w = tpi * fc[i] / tp;
	    pv = (p[ir] * v[ir] + p[ii] * v[ii]) * sc;
	    ps = (p[ir] * p[ir] + p[ii] * p[ii]) * sc;
	    vs = (v[ir] * v[ir] + v[ii] * v[ii]) * sc;
	    vp = atan2(v[ii], v[ir]) / tpi;
	    pm = sqrt(ps);
	    vm = sqrt(vs);
	    fprintf(fp2, "  %12.3e", pv);
	    fprintf(fp3, "  %12.3e", vm);
	    fprintf(fp4, "  %12.3e", vm / w);
	    fprintf(fp5, "  %12.3e", pm);
	    fprintf(fp6, "  %12.3e", vp);
	    if (j == j1) {
	    	pso[i] = ps;
	    }
	}
	
	fprintf(fp2, "\n");
	fprintf(fp3, "\n");
	fprintf(fp4, "\n");
	fprintf(fp5, "\n");
	fprintf(fp6, "\n");
    }

    fprintf(fp4, "; |pst| = ");
    for (i = 0; i < nc; i++)
        fprintf(fp4, "  %12.3e", sqrt(pso[i]));
    fprintf(fp4, "\n");

    fclose(fp2);
    fclose(fp3);
    fclose(fp4);
    fclose(fp5);
    fclose(fp6);

    free(a);
    free(b);
    exit(0);
}
