/* tdm.c - main program for time domain model */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "tdmdef.h"
#define MAIN
#include "cparam.h"
#include "cstate.h"
#include "cimped.h"
#include "cmatrx.h"
#include "ctdmio.h"
#include "cmider.h"
#include "cdata.h"

static int calflg = 0;
static int clpflg = 0;
static int dpeflg = 0;
static int fnmflg = 0;
static int iniflg = 0;
static int logflg = 0;
static int lvlflg = 0;
static int parflg = 0;
static int stmflg = 0;
static int zerflg = 0;
static char ifn[80] = "tdm.par";
static char wfn[80] = "";

int  stmfil();
void alloc_arrays();
void calstm();
void error();
void free_arrays();
void lvlchk();
void lvlrep();
void mechcp();
void mechfl();
void newext();
void parline();
void readpa();
void readst();
void restore();
void saveda();
void savest();
void state0();
void stmini();
void tdmarg();
void tdmdef();
void tdmdfp();
void tdmdfp_man();
void tdmdpe();
void initdm();
void timstp();
void usage();

int
main(int ac, char **av)
{
    tdmdef();		/* set defaults */
    tdmarg(ac, av);	/* interpret command line args */
    initdm(ac, av);	/* initialize state */
    alloc_arrays();	/* allocate memory for arrays */
    mechcp();		/* compute additional mechanical parameters */
    mechfl();		/* initialize fluid mechanics */
    timstp();		/* step through time */
    lvlrep();		/* report level check */
    if (dpeflg)
        tdmdpe();       /* process DPE conditions? */
    free_arrays();	/* free memory for arrays */
    restore();          /* switch back to text mode */
    return(0);		/* done */
}

void
tdmarg(int ac, char **av)
{
    int c, i;

    while (ac > 1) {
	if (av[1][0] == '-') {
	    switch (av[1][1]) {
	    case 'c':
		calflg = 1;
		break;
	    case 'C':
		dpeflg = 1;
		*efn = '\0';
		break;
	    case 'd':
		c = av[1][2];
		if (c >= '1' && c <= '8')
		    for (i = 0; i < (c - '0'); i++)
			dbgflg[i] = 1;
		break;
	    case 'e':
                if (ac > 2) {
                    strcpy(efn, av[2]);
                    dpeflg = 1;
                    ac--;
                    av++;
                }
		break;
	    case 'f':
		fnmflg = 1;
		break;
	    case 'g':
		grfflg = 1;
		break;
	    case 'h':
		usage();
	    case 'i':
		iniflg = 1;
		break;
	    case 'k':
		kbdflg = 1;
		break;
	    case 'l':
		lvlflg = 1;
		break;
	    case 'L':
		logflg = 1;
		break;
	    case 'm':
		tdmdfp_man();
		break;
	    case 'z':
		zerflg = 1;
		break;
	    }
	} else if(strchr(av[1], '=') != NULL) {
	    clpflg = 1;
	} else {
	    if (stmfil(av[1])) {
		strcpy(wfn, av[1]);
		stmflg = 1;
            } else {
		strcpy(ifn, av[1]);
		newext(ifn, ".par");
		parflg = 1;
	    }
	    iniflg = 1;
	}
	ac--;
	av++;
    }
}

void
clpar(ac, av)
char  **av;
{
    while (ac > 1) {
	if (av[1][0] != '-' && strchr(av[1], '=') != NULL)
	    parline(av[1]);
	ac--;
	av++;
    }
}

void
tdmdef()
{
    int i;

    strcpy(sfn, "tdm.sta");
    strcpy(dfn, "tdm.tsd");
    strcpy(lfn, "tdm.log");
    strcpy(cfn, "tdm.cal");
    lfp = stdout;
    grfflg = 0;
    brkflg = 0;
    kbdflg = 0;
    fnmflg = 0;
    wridat = 1;
    for (i = 0; i < 8; i++)
	dbgflg[i] = FALSE;
    pp = (float *) NULL;
    aaa = (float *) NULL;

    tdmdfp();	/* set default parameters */
    state0();	/* initialize state variables to zero */
}

void
initdm(ac, av)
char  **av;
{
    char msg[80];

    if (fnmflg) {
	strcpy(sfn, ifn);
	newext(sfn, ".sta");
	strcpy(dfn, ifn);
	newext(dfn, ".tsd");
	strcpy(lfn, ifn);
	newext(lfn, ".log");
    }
    if (logflg) {
        if((lfp = fopen(lfn, "w+")) == NULL) {
	    sprintf(msg, "can't create log file '%s'\n", lfn);
	    error(msg);
        }
    }
    if (parflg)
	readpa(ifn);		/* read parameter file */
    else if (!stmflg)
	readst(zerflg);		/* read state from file */
    if (stmflg) {
	strcpy(pa_.stimfn, wfn);
	stmini();		/* initialize stimulus timing */
    }
    if (lvlflg)
	lvlchk();		/* adjust parameters for level check */
    if (dpeflg) {
        if (*efn && access(efn, 0) != 0) {
            sprintf(msg, "can't find %s\n", efn);
            error(msg);
        }
        pa_.ifrmt = 8;      /* adjust format for dpe */
        pa_.f1 = 0;         /* no tones for dpe cal */
        zerflg = 1;
        fprintf(lfp, "calibrating\n");
    }
    if (clpflg)
	clpar(ac, av);		/* read parmeters on command line */
    if (zerflg)
	state0();		/* initialize state variables to zero */
    if (calflg)
	calstm();		/* calibrate stimulus tones */

    if (iniflg) {
	umask(0);
	if((sfp = fopen(sfn, "wb")) == NULL) {
	    sprintf(msg, "can't create state file '%s'\n", sfn);
	    error(msg);
	}
	savest();
	fclose(sfp);
	if((dfp = fopen(dfn, "wb")) == NULL) {
	    sprintf(msg, "can't create data file '%s'\n", dfn);
	    error(msg);
	}
	saveda();
	if (zerflg) {
	    for (pa_.itsf = 1; pa_.itsf < pa_.ntsf; pa_.itsf++)
                saveda();
            pa_.itsf = 0;
        }
	fclose(dfp);
    }
}

void
lvlchk()
{
    zerflg = 1;
    pa_.n = 1;
    wridat = 0;		/* supress data writing */
    fprintf(lfp, "level check on pressure at eardrum\n");
}

void
lvlrep()
{
    float pmax, spl, psum, pssq, prms, pmsq, n;
    static float pref = 2e-4; 	/* dyn/cm^2 */

    if ((n = st_.pednum) > 0) {
	pmax = st_.pedmax;
	psum = st_.pedsum;
	pssq = st_.pedssq;

	spl = 20.0 * log10(pmax / pref);
	fprintf(lfp, "peak pressure at eardrum = %.3e dyn/cm^2  ", pmax);
	fprintf(lfp, "= %4.1f dB pSPL\n", spl);

	pmsq = psum * psum / n;
	prms = sqrt((pssq - pmsq) / n);
	spl = 20.0 * log10(prms / pref);
	fprintf(lfp, " rms pressure at eardrum = %.3e dyn/cm^2  ", prms);
	fprintf(lfp, "= %4.1f dB SPL\n", spl);
    }
}

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}

void
error(s)
char   *s;
{
    free_arrays();	/* free memory for arrays */
    restore(); 		/***** resume text mode ****/
    fprintf(stderr, "ERROR: %s\n", s);
    exit(1);
}

void
usage()
{
    puts("usage:");
    puts("    tdm [-options] [parameter_file]");
    puts("options:");
    puts("    c   - use calibration file to adjust tone levels");
    puts("    C   - create a calibration file");
    puts("    dN  - turn on debug flags 1 through N");
    puts("    e N - process DPE list file N");
    puts("    f   - use the parameter file name for data files also");
    puts("    g   - display BM displacement graphically");
    puts("    h   - print this usage information");
    puts("    i   - create new data files");
    puts("    k   - check keyboard");
    puts("    l   - level check");
    puts("    m   - set 'man' parameters");
    puts("    z   - set state to zero");
    exit(0);
}

static void
caltone(float *f, float *l, double tp)
{
    double c, fp, fc;

    if (*f > 0 && tp > 0) {
        fp = floor(*f * tp + 0.5) / tp;
        cfp = fopen(cfn, "r");
        if (cfp != NULL) {
            while(fscanf(cfp, " %lf %lf ", &fc, &c) > 0) {
                if ((abs(fp - fc) * tp) <= 0.5) {
                    *l += c;
                    break;
                }
            }
            fclose(cfp);
        }
        *f = fp;
    }
}

void
calstm()
{
    double tp;
    
    printf("  L1,L2,L3,L4         = %8.0f%8.0f%8.0f%8.0f\n",
	pa_.l1, pa_.l2, pa_.l3, pa_.l4);
    tp = pa_.ntsf * pa_.ntsw * pa_.dt;
    caltone(&pa_.f1, &pa_.l1, tp);
    caltone(&pa_.f2, &pa_.l2, tp);
    caltone(&pa_.f3, &pa_.l3, tp);
    caltone(&pa_.f4, &pa_.l4, tp);
}

void
alloc_arrays()
{
    if (pp)
        free(pp);
    if (aaa)
        free(aaa);
    pp = (float *) calloc(Psize, sizeof(float));
    aaa = (float *) calloc(Asize, sizeof(float));
    aa1 = aaa;
    aa2 = aaa + Nmax;
    aa3 = aaa + Nmax * 2;
}

void
free_arrays()
{
    if (pp)
        free(pp);
    if (aaa)
        free(aaa);
}

