/* tdmdio.c - data input and output subroutines for time-domain model */

#include <stdio.h>
#include "tdmdef.h"
#include "tdmver.h"
#include "cparam.h"
#include "cdata.h"
#include "cmider.h"
#include "ctdmio.h"
#include "cstate.h"

#define limit(a,b,c)	(((a)>(b))?(a):((c)<(b))?(c):(b))

void alloc_arrays();
void error();
void getdsav();
void saveda();
void state0();
void swab();		/* libc function */
void type_graf();
void vcen();
void vmov();
void vsub();
void vzero();

extern float *pp;

static float wnr;
static int dwr = 0;

/* wrdata - write data */

void
wrdata()
{
    float   cen;
    float   *dme, *ddme, *d1, *d2;

    dme = st_.d_;
    ddme = st_.dd_ + ST_OFF_ME;
    d1 = st_.d_ + ST_OFF_1;
    d2 = st_.d_ + ST_OFF_2;

    if (pa_.ifrmt == 0) {	/* format - 0 */
	getdsav();
	type_graf();
    } else if (pa_.ifrmt == 1) {/* format - 1 */
	getdsav();
	saveda();
    } else if (pa_.ifrmt == 2) {/* format - 2 */
	getdsav();
	saveda();
    } else if (pa_.ifrmt == 8) {/* format - 8 */
	getdsav();
	vcen(dsav, &cen, pa_.n);
	rsav[0] = vep;
	rsav[1] = cen;
	rsav[2] = wnr;
	rsav[3] = dsav[pa_.isv[4]];
	rsav[4] = ped;
	rsav[5] = dme[ME_MA];
	rsav[6] = ddme[ME_ED] * pa_.acp;
	rsav[7] = ped * ddme[ME_ED] * pa_.acp;
	rsav[8] = pst;
	rsav[9] = dme[ME_ST];
	rsav[10] = ddme[ME_ST] * pa_.ast;
	rsav[11] = pst * ddme[ME_ST] * pa_.ast;
	rsav[12] = dsav[pa_.isv[0]];
	rsav[13] = dsav[pa_.isv[1]];
	rsav[14] = dsav[pa_.isv[2]];
	rsav[15] = dsav[pa_.isv[3]];
	saveda();
    } else if (pa_.ifrmt == 9) {/* format - 9 */
	getdsav();
	vcen(dsav, &cen, pa_.n);
	rsav[0] = vep;
	rsav[1] = ped;
	rsav[2] = pst;
	rsav[3] = wnr;
	rsav[4] = d1[pa_.isv[0]];
	rsav[5] = d1[pa_.isv[1]];
	rsav[6] = d1[pa_.isv[2]];
	rsav[7] = d1[pa_.isv[3]];
	rsav[8] = d1[pa_.isv[0]] - d2[pa_.isv[0]];
	rsav[9] = d1[pa_.isv[1]] - d2[pa_.isv[1]];
	rsav[10] = d1[pa_.isv[2]] - d2[pa_.isv[2]];
	rsav[11] = d1[pa_.isv[3]] - d2[pa_.isv[3]];
	rsav[12] = st_.vohc[pa_.isv[0]];
	rsav[13] = st_.vohc[pa_.isv[1]];
	rsav[14] = st_.vohc[pa_.isv[2]];
	rsav[15] = st_.vohc[pa_.isv[3]];
	saveda();
    }
}

void
type_graf()
{
    char    line[81];
    double  dmxmg, time, disp, dxmm, dist, xn, sc;
    int     imxmg, i, j, k, l, nl, nc;

    /* find maximum bm displacement */
    dmxmg = 0.0;
    imxmg = 0;
    for (i = 0; i < pa_.n; i++) {
	if (dmxmg < dsav[i]) {
	    dmxmg = dsav[i];
	    imxmg = i;
	} else if (dmxmg < -dsav[i]) {
	    dmxmg = -dsav[i];
	    imxmg = i;
	}
    }
    /* go to new page and write header info */
    time = (pa_.itsc + pa_.icyc * pa_.ntsc) * pa_.dt * 1e3;
    disp = dmxmg * 1e7;
    dxmm = pa_.xl * 10.0 / (pa_.n - 1);
    dist = imxmg * dxmm;
    fprintf(lfp, "\f");
    fprintf(lfp, " At time = %8.3f ms \n", time);
    fprintf(lfp, " maximum bm displacement is %8.3f nm\n", disp);
    fprintf(lfp, " at position %7.3f mm from stapes.\n", dist);
    fprintf(lfp, "\n");
    /* plot bm displacement vs. distance from stapes */
    nl = 60;		/* nl = number of lines */
    nc = 40;		/* nc = number of columns */
    xn = (pa_.n - 1.0) / (nl - 1.0);
    sc = (dmxmg > 0) ? 1.0 / dmxmg : 1.0;
    for (i = 0; i < nl - 1; i++) {
	for (j = 0; j < nc; j++)
	    line[j] = ' ';
	for (k = (i * xn); k <= ((i + 1) * xn); k++) {
	    l = nc * limit(0.0, (0.5 * dsav[k] * sc + 0.5), 1.0);
	    line[l] = '*';
	}
	line[nc] = '\0';
	dist = i * xn * dxmm;
	fprintf(lfp, "%6.2f - %s\n", dist, line);
    }
}

void
getdsav()
{
    char    line[81];
    float   *d1, *d2, *dd1, *dd2;

    d1 = st_.d_ + ST_OFF_1;
    d2 = st_.d_ + ST_OFF_2;
    dd1 = st_.dd_ + ST_OFF_1;
    dd2 = st_.dd_ + ST_OFF_2;
    switch (pa_.jfrmt) {
    case 0:
    case 1:
	vmov(d1, dsav, pa_.n);
	break;
    case 2:
	vsub(d1, d2, dsav, pa_.n);
	break;
    case 3:
	vmov(st_.cc, dsav, pa_.n);
	break;
    case 4:
	vmov(dd1, dsav, pa_.n);
	break;
    case 5:
	vsub(dd1, dd2, dsav, pa_.n);
	break;
    case 6:
	vmov(st_.sc, dsav, pa_.n);
	dsav[0] = st_.wnap;
	break;
    default:
	sprintf(line, "can't do jfrmt=%d\n", pa_.jfrmt);
	error(line);
    }
    wnr = st_.wnap;
    st_.wnap = 0;
    vzero(st_.sc, pa_.n);
}

/* saveda - write to data file */

void
saveda()
{
    int i, m, n;
    long dsiz, hsiz;

    /* write parameter block */
    pa_.magic = 0x74736400;	/* "tsd" */
    pa_.ireor = 0;
    fseek(dfp, 0L, 0);
    if (fwrite((void *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	error("writing data (header)\n");
    }

    /* write data */
    hsiz = pa_.hdrsiz;
    if (pa_.ifrmt <= 0) {
	return;
    } else if (pa_.ifrmt == 1) {
	dsiz = pa_.n * sizeof(float);
	fseek(dfp, hsiz + pa_.itsf * dsiz, 0);
	if (fwrite((void *) dsav, dsiz, 1, dfp) < 1)
	    error("writing data (dsav)\n");
    } else if (pa_.ifrmt == 2) {
	m = pa_.m;
	n = pa_.n;
	dsiz = sizeof(float) * n;
	fseek(dfp, hsiz + pa_.itsf * dsiz * 2, 0);
	if (pp == NULL)
	    alloc_arrays();
	if (pa_.m > 1) {
	    for (i = 1; i < n; i++)
	        pp[i] = pp[i * m];	/* overwrite pp */
	}
	if (fwrite((void *) pp, dsiz, 1, dfp) < 1)
	    error("writing data (pp)\n");
	if (fwrite((void *) dsav, dsiz, 1, dfp) < 1)
	    error("writing data (dsav)\n");
    } else if (pa_.ifrmt == 3 || pa_.ifrmt == 7 || pa_.ifrmt == 8) {
	dsiz = 16 * sizeof(float);
	fseek(dfp, hsiz + pa_.itsf * dsiz, 0);
	if (fwrite((void *) rsav, dsiz, 1, dfp) < 1)
	    error("writing data (rsav)\n");
    }
    dwr++;
}

/* sswab - swap bytes (in place) for n "short ints" */
void
sswab(short *p, int n)
{
    swab(p, p, n * 2); 
}

/* lswab - swap bytes (in place) for n "long ints" */
void
lswab(long *p, int n)
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) p++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

/* hdr_swab - swap header bytes  */
void
hdr_swab()
{
    sswab((short *) &pa_.hdrsiz, 6);
    lswab((long *) &pa_.dt, 14);
    sswab((short *) &pa_.m, 21);
    lswab((long *) &pa_.xl, 168);
}

/* readst - read state and parameters  */

void
readst(int zerflg)
{
    int  swap_bytes = 0;
    FILE *sfp;

    sfp = fopen("tdm.sta", "rb");

    /* read header */
    fseek(sfp, 0L, 0);
    if (fread((char *) &pa_, sizeof(pa_), 1, sfp) < 1)
	error("reading state (header)");
    if (pa_.ifrmt > 255) {
        hdr_swab();
	swap_bytes++;
    }

    /* read state */
    if (zerflg) {
	state0();
    } else {
        fseek(sfp, (long) pa_.hdrsiz, 0);
        if (fread((char *) &st_, sizeof(st_), 1, sfp) < 1)
	    error("reading state");
        if (swap_bytes)
	    lswab((long *) st_.d_, NSTVmax * 2 + Nmax * 5 + 7 );
    }

    fclose(sfp);
}

/* savest - save state and parameters  */

void
savest()
{
    long hsiz;

    pa_.magic = 0x73746100;	/* "sta" */
    pa_.hdrsiz = 1024;
    pa_.vernum = VERNUM;
    pa_.revnum = REVNUM;
    pa_.ireor = -1;

    /* write parameter block to state file */
    fseek(sfp, 0L, 0);
    if (fwrite((void *) &pa_, sizeof(pa_), 1, sfp) < 1) {
	error("writing state (header)\n");
    }
    hsiz = pa_.hdrsiz;
    if (hsiz < sizeof(pa_))
	error("too many parameters!");
    fseek(sfp, hsiz, 0);
    if (fwrite((void *) &st_, sizeof(st_), 1, sfp) < 1) {
	error("writing state\n");
    }

    fprintf(lfp, "  ntsc,ncyc,ntsw,ntsf = %8ld%8ld%8ld%8ld\n",
    pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    fprintf(lfp, "  itsc,icyc,itsw,itsf = %8ld%8ld%8ld%8ld\n",
    pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);

    fflush(sfp);
    fflush(lfp);
    if (dwr) {
        fflush(dfp);
        dwr = 0;
    }
}
