/* tdmdpe.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "tdmdef.h"
#include "cparam.h"
#include "ctdmio.h"
#include "cdata.h"

#define MAXNDAT 64

void calstm();
void dathdr();
void error();
void dpelev();
void mkcal();
void newext();
void rfft();
void state0();
void timstp();

static char xfn[80] = "tdm.dat";
static char msg[80];
static short datfmt = 0, ndat = 0;
static long t1;
static FILE *xfp;

void
tdmdpe()
{
    char s[256];
    double f1, f2, f3, f4, l1, l2, l3, l4, tm, ns, sn;
    int    n, hdrflg = 0;
    static char *fmtpar = ";datfmt=";
    
    mkcal();
    if (*efn == 0)
        return;
    if ((efp = fopen(efn, "r")) == NULL) {
	sprintf(msg, "can't open %s\n", efn);
	error(msg);
    }
    fprintf(lfp, "opened DPE file: %s\n", efn);
    strcpy(xfn, efn);
    newext(xfn, ".dat");
    if ((xfp = fopen(xfn, "w")) == NULL) {
	sprintf(msg, "can't open %s\n", xfn);
	error(msg);
    }
    pa_.ntsgat = floor(0.020 / pa_.dt + 0.5);    /* 20 ms on ramp */
    if (pa_.id1 > 0)
	pa_.isv[1] = pa_.id1;
    pa_.ncyc = pa_.ncdp;
    datfmt = 0;
    while (fgets(s, 256, efp) != NULL) {
        if (*s == ';') {
            n = strlen(fmtpar);
            if (strncmp(s, fmtpar, n) == 0) {
                datfmt = s[n] - '0';
            }
        } else if (*s == 0) {
            continue;
        } else {
            if (hdrflg == 0) {
                dathdr();
                hdrflg = 1;
            }
            t1 = time(0L);
            f1 = f2 = f3 = f4 = 0;
            l1 = l2 = l3 = l4 = 0;
            sscanf(s, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
		&f2, &f1, &l2, &l1, &tm, &ns, &sn, &f3, &l3, &f4, &l4);
            pa_.f1 = f1;
            pa_.f2 = f2;
            pa_.f3 = f3;
            pa_.f4 = f4;
            pa_.l1 = l1;
            pa_.l2 = l2;
            pa_.l3 = l3;
            pa_.l4 = l4;
            calstm();
            state0();
            brkflg = 0;
            timstp();
	    if (pa_.icyc != pa_.ncyc) {
		printf("aborting\n");
		break;
	    }
            dpelev();
            fflush(xfp);
        }
    }
    fclose (efp);
    fclose (xfp);
}

void
dathdr()
{
    int    i;
    static char *datlab[12] = {
    	";  f2 ", "   f1 ", "   L2 ", "   L1 ", "   ET ", 
    	"   Ldp ", "   Pst ", "   <dp ", "   D0  ", "   D1  ", 
    	"   D2  ", "   WNR "
    };
    static char *dlabel[5] = { 
        "    Ped    <Ped     Pst     D0      D1      D2  ",
        "|=======|",
        "   WNR   ",
        "|=======================|",
        "   F3      F4      WNR   ",
    };
    static char *dathdr[9] = { 
        ";=================================",
        "|====================  f1 ======================",
        "|====================  f2 ======================",
        "|=================  2*f1-f2 ====================",
        "|=================  3*f1-2*f2 ==================",
        "|================== 4*f1-3*f2 ==================",
        "|===================  f2-f1 ====================",
        "|====================  f3 ======================",
        "|====================  f4 ======================",
    };
    
    if (datfmt == 1) {
        ndat = 5 + 6 * 6 + 1;
        for (i = 0; i <= 6; i++)
            fprintf(xfp, "%s", dathdr[i]);
        fprintf(xfp, "%s\n", dlabel[1]);
        for (i = 0; i < 5; i++)
            fprintf(xfp, "%s ", datlab[i]);
        for (i = 0; i < 6; i++)
            fprintf(xfp, "%s", dlabel[0]);
        fprintf(xfp, "%s", dlabel[2]);
    } else if (datfmt == 2) {
        ndat = 5 + 6 * 8 + 3;
        for (i = 0; i <= 8; i++)
            fprintf(xfp, "%s", dathdr[i]);
        fprintf(xfp, "%s\n", dlabel[3]);
        for (i = 0; i < 5; i++)
            fprintf(xfp, "%s ", datlab[i]);
        for (i = 0; i < 8; i++)
            fprintf(xfp, "%s", dlabel[0]);
        fprintf(xfp, "%s", dlabel[4]);
    } else {
        ndat = 5 + 7;
        for (i = 0; i < ndat; i++)
            fprintf(xfp, "%s ", datlab[i]);
    }
    fprintf(xfp, "\n");
}

static double
dbv(double v)
{
    return((v < 1e-20) ? -999 : 20 * log10(v));
}

static double
log_10(double v)
{
    return((v < 1e-20) ? -999 : log10(v));
}

void
mkcal()
{
    double  tp, sc, vep, ped;
    int     nrows, ncols, npts, i, j, ir, ii, nf;
    long    hsiz;
    float  *a, *b, *c, cmg, cph;
    static double spl_ref = 0.0002 * 1.4142;

    if ((dfp = fopen(dfn, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", dfn);
	error(msg);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	error("reading header\n");
    }
    if (pa_.ifrmt != 8 || pa_.ireor != 0 || pa_.f1 != 0) {
	sprintf(msg, "wrong type of data file: ifrmt=%d ireor=%d f1=%.0f\n", 
	    pa_.ifrmt, pa_.ireor, pa_.f1);
	error(msg);
    }
    ncols = pa_.ntsf;
    nrows = 16;
    npts = nrows * ncols;

    a = (float *) malloc(nrows * sizeof(float));
    b = (float *) malloc(nrows * ncols * sizeof(float));
    if (a == NULL || b == NULL) {
	error("allocating memory\n");
    }
    hsiz = pa_.hdrsiz;
    fseek(dfp, hsiz, 0);
    for (i = 0; i < ncols; i++) {
	if (fread((char *) a, sizeof(float), nrows, dfp) < nrows) {
	    /* error("reading data\n") */;
	}
	for (j = 0; j < nrows; j++)
	    b[i + j * ncols] = a[j];
    }
    fclose(dfp);

    nf = ncols / 2;
    sc = (2.0 / ncols);
    for (j = 0; j < nrows; j++) {
        c = b + j * ncols;
        rfft(c, ncols);
        for (i = 1; i < nf; i++) {
            ir = 2 * i;
            ii = 2 * i + 1;
            cmg = hypot(c[ii],c[ir]) * sc;
            cph = atan2(c[ii],c[ir]);
            c[ir] = cmg;
            c[ii] = cph;
	}
    }

    for (i = 0; i < ncols; i++) {
        for (j = 0; j < nrows; j++)
            a[j] = b[i + j * ncols];
    }
    cfp = fopen(cfn, "w");
    c = b + 4 * ncols;
    tp = pa_.ntsf * pa_.ntsw * pa_.dt;
    for (i = 1; i < nf; i++) {
        ir = 2 * i;
        vep = dbv(b[ir] / pa_.stgain);
        ped = dbv(c[ir] / spl_ref);
        fprintf(cfp, "%6.0f %7.2f\n", i / tp, vep - ped);
    }

    fclose(cfp);
    fprintf(stderr, "wrote CAL file: %s\n", cfn);
    free(a);
    free(b);
}

void
dpelev()
{
    double  dat[MAXNDAT];
    double  tp, sc, ped, pst, sv0, sv1, sv2, wnr;
    int     nrows, ncols, npts, i, j, ir, ii, nf;
    int     fc[32], nc = 0;
    long    hsiz;
    float  *a, *b, *c, cmg, cph;
    static double rad2deg = 180 / 3.1415927;
    static double dsp_ref = 1e-10;
    static double spl_ref = 0.0002 * 1.4142;

    if ((dfp = fopen(dfn, "rb")) == NULL) {
	sprintf(msg, "can't open %s\n", dfn);
	error(msg);
    }
    if (fread((char *) &pa_, sizeof(pa_), 1, dfp) < 1) {
	error("reading header\n");
    }
    if (pa_.ifrmt != 8 || pa_.ireor != 0 || pa_.f1 <= 0) {
	sprintf(msg, "wrong type of data file: ifrmt=%d ireor=%d f1=%.0f\n", 
	    pa_.ifrmt, pa_.ireor, pa_.f1);
	error(msg);
    }
    ncols = pa_.ntsf;
    nrows = 16;
    npts = nrows * ncols;

    a = (float *) malloc(nrows * sizeof(float));
    b = (float *) malloc(npts * sizeof(float));
    if (a == NULL || b == NULL) {
	error("allocating memory\n");
    }
    hsiz = pa_.hdrsiz;
    fseek(dfp, hsiz, 0);
    for (i = 0; i < ncols; i++) {
	if (fread((char *) a, sizeof(float), nrows, dfp) < nrows) {
	    /* error("reading data\n") */;
	}
	for (j = 0; j < nrows; j++)
	    b[i + j * ncols] = a[j];
    }
    fclose(dfp);

    nf = ncols / 2;
    sc = (2.0 / ncols);
    for (j = 0; j < nrows; j++) {
        c = b + j * ncols;
        rfft(c, ncols);
        for (i = 1; i < nf; i++) {
            ir = 2 * i;
            ii = 2 * i + 1;
            cmg = hypot(c[ii],c[ir]) * sc;
            cph = (cmg > 0) ? atan2(c[ii],c[ir]) : 0;
            c[ir] = cmg;
            c[ii] = cph;
	}
    }
    wnr = log_10(b[0 + 2 * ncols] / ncols);

    tp = pa_.ntsf * pa_.ntsw * pa_.dt;
    nc = 0;
    fc[nc++] = pa_.f1 * tp + 0.5;
    fc[nc++] = pa_.f2 * tp + 0.5;
    fc[nc++] = 2 * fc[0] - fc[1];
    if (datfmt == 1) {
        fc[nc++] = 3 * fc[0] - 2 * fc[1];
        fc[nc++] = 4 * fc[0] - 3 * fc[1];
        fc[nc++] = fc[1] - fc[0];
    } else if (datfmt == 2) {
        fc[nc++] = 3 * fc[0] - 2 * fc[1];
        fc[nc++] = 4 * fc[0] - 3 * fc[1];
        fc[nc++] = fc[1] - fc[0];
        fc[nc++] = pa_.f3 * tp + 0.5;
        fc[nc++] = pa_.f4 * tp + 0.5;
    }
    for (i = 0; i < nc; i++) {
	ir = 2 * fc[i];
	ii = ir + 1;
	if (ir > 0 && ii < ncols) {
	    for (j = 0; j < nrows; j++)
		a[j] = b[ir + j * ncols];
	} else {
	    for (j = 0; j < nrows; j++)
		a[j] = 0;
	}
	ped = dbv(a[4] / spl_ref);
	pst = dbv(a[8] / spl_ref);
	sv0 = dbv(a[12] / dsp_ref);
	sv1 = dbv(a[13] / dsp_ref);
	sv2 = dbv(a[14] / dsp_ref);
	if (ir > 0 && ii < ncols) {
	    for (j = 0; j < nrows; j++)
		a[j] = b[ii + j * ncols];
	}
	if (i == 0) {
	    dat[1] = fc[0] / tp;
	    dat[3] = ped;
	} else if (i == 1) {
	    dat[0] = fc[1] / tp;
	    dat[2] = ped;
	} else if (i == 2) {
	    dat[4] = time(0L) - t1;
	}
	if (datfmt == 1) {
	    dat[i * 6 + 5] = ped;
	    dat[i * 6 + 6] = a[4] * rad2deg;
	    dat[i * 6 + 7] = pst;
	    dat[i * 6 + 8] = sv0;
	    dat[i * 6 + 9] = sv1;
	    dat[i * 6 + 10] = sv2;
	    dat[i * 6 + 11] = wnr;
	} else if (datfmt == 2) {
	    dat[i * 6 + 5] = ped;
	    dat[i * 6 + 6] = a[4] * rad2deg;
	    dat[i * 6 + 7] = pst;
	    dat[i * 6 + 8] = sv0;
	    dat[i * 6 + 9] = sv1;
	    dat[i * 6 + 10] = sv2;
	    dat[i * 6 + 11] = pa_.f3;
	    dat[i * 6 + 12] = pa_.f4;
	    dat[i * 6 + 13] = wnr;
	} else {
	    if (i == 0) {
		dat[9] = sv1;
	    } else if (i == 1) {
		dat[8] = sv0;
	    } else if (i == 2) {
		dat[5] = ped;
		dat[6] = pst;
		dat[7] = a[4] * rad2deg;
		dat[10] = sv2;
		dat[11] = wnr;
	    }
	}
    }
    fprintf(xfp, "%6.0f %6.0f %6.1f %6.1f %6.0f ", 
        dat[0], dat[1], dat[2], dat[3], dat[4]);
    for (i = 5; i < ndat; i++)
        fprintf(xfp, "%7.2f ", dat[i]);
    fprintf(xfp, "\n");
    free(a);
    free(b);
    fprintf(stderr, "  --> DAT file: %s\n", xfn);
}
