/* tdmdsp.c - time-step display for time-domain model */

#include <stdio.h>
#include <stdlib.h>
#include "tdmdef.h"
#include "cparam.h"
#include "ctdmio.h"
#include "cstate.h"
#include "cdata.h"

#ifdef MSDOS
#include <pc.h>
#else /* MSDOS */
#include <unistd.h>
#endif /* MSDOS */

#ifdef GRX 
#include <string.h>
#include <grdriver.h>
#include <grx20.h>
#include <grdriver.h>
#include <grfontdv.h>
#define YP
#endif /* GRX */

#ifdef XW
void xw_init(int *, int *);
void xw_line(int, int, int, int, int, int);
void xw_quit();
#undef YP
#define YP
#endif /* XW */

void error();

static int calledyet = 0;
static int sd = 8;
static int grfmod = 0;
#ifdef YP
static int yp[Nmax] = {0};
#endif

void
keyboard()
{
#ifdef GRX
    if (kbhit()) {
        int c;

        c = getkey();
        if (c == 27)
            brkflg++;
        else if (c == 'g')
	    grfflg = 1;
        else if (c >= '1' && c <= 'd')
            sd = 2 * (c - '0');
    }
#endif /* GRX */
}

void
display()
{
    float d1mxmg, xe;
    float *d1;
    float xg, yg, xo, yo;
    float dx, dy;
    float d1mnfs = 1e-20;
    int i, nn, xx1, yy1, ypp, b1, b2;
#ifdef YP
    int xx, yy;
#endif
#ifdef GRX
    static int c0, c1;
#endif /* GRX */
    static int xs, ys, dropped = 0;

    if (kbdflg || grfflg)
	keyboard();
    if (grfflg & !calledyet) {
#ifdef XW
        if (getenv("DISPLAY"))
	    grfmod = 2;
#endif /* XW */
#ifdef GRX
        if (!grfmod)
	    grfmod = 1;
#endif /* GRX */
    }
    if (!dropped && grfmod != 1) {
#ifndef MSDOS
	setuid(getuid());		/* drop root priveleges */
	setgid(getgid());
#endif /* MSDOS */
	dropped++;
    }
    if (!grfflg) {
        return;
    }
    if (!calledyet) {
#ifdef GRX
        if (grfmod == 1) { 
            if (lfp == stdout) {
	        if((lfp = fopen(lfn, "w+")) == NULL) {
		    char msg[80];
		    sprintf(msg, "can't create log file '%s'\n", lfn);
		    error(msg);
	        }
	    }
            GrSetMode(GR_default_graphics);		
            c0 = GrWhite();
            c1 = GrBlack();
            xs = GrSizeX();
            ys = GrSizeY();
            GrFilledBox(0, 0, xs, ys, c0);
	}
#endif /* GRX */
#ifdef XW
        if (grfmod == 2) {
            xw_init(&xs, &ys);
        }
#endif /* XW */
	calledyet++;
    }

    d1 = st_.d_ + ST_OFF_1;
    nn = pa_.n;
    
/* Get the magnitude */
    d1mxmg = 0;
    for (i = 0; i < pa_.n; i++) {
	if (d1[i] != d1[i])		/* check for NaN */
	    continue;
	if (d1mxmg < d1[i]) {
	    d1mxmg = d1[i];
	} else if (d1mxmg < -d1[i]) {
	    d1mxmg = -d1[i];
	}
    }
    dx = xs;
    dy = ys / sd - 1;
    xo = 0.0;
    yo = ys / 2;
    xx1 = xo;
    yy1 = yo;
    ypp = yo;
    xg = dx / nn;
    yg = (d1mxmg > d1mnfs) ? dy / d1mxmg : dy / d1mnfs;
    xe = 0;
    b1 = yo - dy - 1;
    b2 = yo + dy;

#ifdef GRX
    if (grfmod == 1) {
        for (i = 0; i < nn; i++) {
            xx = xo + xg * i;
            yy = yo - yg * d1[i];
            GrLine(xx1, ypp, xx, yp[i], c0);
            GrLine(xx1, yy1, xx, yy, c1);
            xx1 = xx;
            yy1 = yy;
            ypp = yp[i];
            yp[i] = yy;
        }
    }
#endif /* GRX */
#ifdef XW
    if (grfmod == 2) {
        for (i = 0; i < nn; i++) {
            xx = xo + xg * i;
            yy = yo - yg * d1[i];
            xw_line(xx1, ypp, xx, yp[i], yy1, yy);
            xx1 = xx;
            yy1 = yy;
            ypp = yp[i];
            yp[i] = yy;
        }
    }
#endif /* XW */
}

void
restore()
{
    if (calledyet) {
#ifdef GRX
        if (grfmod == 1) {
            GrSetMode(GR_default_text);
	}
#endif /* GRX */
#ifdef XW
        if (grfmod == 2) {
            xw_quit();
        }
#endif /* XW */
        fclose(lfp);
        lfp = stdout;
    }
}

