/* gbe.c - Gaussian block-elimination routines */

#include <stdio.h>

/* vprn - print matrix (for debugging) */

void
vprn(int n, float *x, char *s)
{
    int i;

    fprintf(stderr, "%s ", s);
    for (i = 0; i < n; i++)
        fprintf(stderr, "%8.3f", x[i]);
    fprintf(stderr, "\n");
}

static void
vmov(float *a, float *b, int n)
{
    int i;

    for (i = 0; i < n; i++) {
	a[i] = b[i];
    }
}

static void
vadd(float *a, float *b, float *c, int n)
{
    int i;

    for (i = 0; i < n; i++) {
	a[i] = b[i] + c[i];
    }
}

static void
mmul(float *a, float *b, float *c, int n1, int n2, int n3)
{
    int i, j, k;

    for (i = 0; i < n1; i++) {
        for (k = 0; k < n3; k++) {
	    a[i + k * n1] = 0;
	    for (j = 0; j < n2; j++) {
		a[i + k * n1] += b[i + j * n1] * c[j + k * n2];
	    }
	}
    }
}

/* gbe - vector operations for Gaussian block elimination */

void
gbe(float *a, float *p, float *tk, int m, int n, int basacc, int apxacc)
{
    float *ak, *pk, *qk;
    int k, msq, nm1, nm2;

    msq = m * m;
    nm1 = n - 1;
    nm2 = n - 2;

    if (basacc){			/* stapes acceleration */
        /* matrix-vector multiply ' pk = ak*pk' */
        ak = a;
        pk = p;
        mmul(tk, ak, pk, m, m, 1);
        vmov(pk, tk, m); 
    }

    for(k = 1; k < nm1; k++) {		/* base to apex */
        /* matrix-vector multiply ' pk = ak*( pk + qk ) ' */
        ak = a + k * msq;
        pk = p + k * m;
        qk = pk - m;
        vadd(tk, pk, qk, m);
        mmul(pk, ak, tk, m, m, 1);
   }

    if (apxacc) {			/* rigid wall at apex */
        /* matrix-vector multiply ' pk = ak*( pk + 2*qk ) ' */
        ak = a + nm1 * msq;
        pk = p + nm1 * m;
        qk = pk - m;
        vadd(tk, qk, qk, m);
        vadd(tk, pk, tk, m);
        mmul(pk, ak, tk, m, m, 1);
    }

    for (k = nm2; k > 0; k--) {	/* apex to base */
        /* matrix-vector multiply ' pk = pk + ak*qk ' */
        ak = a + k * msq;
        pk = p + k * m;
        qk = pk + m;
        mmul(tk, ak, qk, m, m, 1);
        vadd(pk, pk, tk, m);
    }

    if (basacc) {			/* stapes acceleration */
        /* matrix-vector multiply ' pk = pk + 2*ak*qk ' */
        ak = a;
        pk = p;
        qk = pk + m;
        mmul(tk, ak, qk, m, m, 1);
        vadd(tk, tk, tk, m);
        vadd(pk, pk, tk, m);
    }

}
