/* tdmpar - parameters for time-domain model from Neely (1981), JASA */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "tdmdef.h"
#include "tdmver.h"
#include "cparam.h"
#include "ctdmio.h"
#include "cdata.h"

void chkpar();
void error();
void parline();

void
tdmdfp_man()
{
   pa_.k1a = 4e8;
   pa_.k1b = 2.099e6;
   pa_.k1c = 1.102e4;
   pa_.k2a = 2e8;
   pa_.k2b = 7.396e5;
   pa_.k2c = 2735;
   pa_.k3a = 2.25e8;
   pa_.k3b = 8.32e5;
   pa_.k3c = 3077;
   pa_.k4a = 4.5e8;
   pa_.k4b = 1.982e6;
   pa_.k4c = 8732;
   pa_.m1a = 0.004;
   pa_.m1b = 0.004;
   pa_.m1c = 0.004;
   pa_.m2a = 0.037;
   pa_.m2b = 0.037;
   pa_.m2c = 0.037;
   pa_.mcp = 2e-4;
   pa_.med = 1e-4;
   pa_.r1a = 960;
   pa_.r1b = 82.84;
   pa_.r1c = 7.149;
   pa_.r2a = 2000;
   pa_.r2b = 172.6;
   pa_.r2c = 14.89;
}

void
tdmdfp()
{

    pa_.acp = 0.071;
    pa_.adi = 2.5;
    pa_.aed = 0.05;
    pa_.ama = 0.35 ;
    pa_.ast = 0.01;
    pa_.bl  = 5.4e6;
    pa_.d3max = 1e-7;
    pa_.dhbmax = 1;
    pa_.dhbmin = 0;
    pa_.dihc = 3e-8;
    pa_.dt = 2e-6;
    pa_.f1 = 0;
    pa_.f2 = 0;
    pa_.f3 = 0;
    pa_.f4 = 0;
    pa_.fpma = 0.1654;
    pa_.fpmb = 0.021;
    pa_.fpmc = 0.99;
    pa_.freq = 1000;
    pa_.gam = 1;
    pa_.gamuns = 1;
    pa_.gmel = 0.5;
    pa_.icyc = 0;
    pa_.id1 = 0;
    pa_.ifrmt = 8;
    pa_.ihceq = 0;
    pa_.ipl = 0;
    pa_.isv[0] = 80;
    pa_.isv[1] = 160;
    pa_.isv[2] = 240;
    pa_.isv[3] = 320;
    pa_.isv[4] = 360;
    pa_.isv[5] = 400;
    pa_.itsc = 0;
    pa_.itsf = 0;
    pa_.itsw = 0;
    pa_.jfrmt = 2;
    pa_.k1r = 0;
    pa_.kcp = 2200;
    pa_.kdi = 7.7e6;
    pa_.ked = 3.1e4;
    pa_.khe = 0.0;
    pa_.kim = 4e7;
    pa_.kma = 4e5;
    pa_.krw = 0;
    pa_.kst = 5e5;
    pa_.l1 = 0;
    pa_.l2 = 0;
    pa_.l3 = 0;
    pa_.l4 = 0;
    pa_.nrgn = 2.5e6;
    pa_.m = 1;
    pa_.mco = 2.25e3;
    pa_.mcp = 2.2e-4;
    pa_.mdi = 5.7e-2;
    pa_.med = 1.4e-4;
    pa_.mhe = 1e-5;
    pa_.mma = 0.01;
    pa_.mmeq = 1;
    pa_.mst = 5.0e-3;
    pa_.n = 401;
    pa_.ncyc = 1;
    pa_.ncdp = 2;
    pa_.nimp = 1;
    pa_.ntsc = 20480;
    pa_.ntsf = 2048;
    pa_.ntsw = 10;
    pa_.ohccmp = 0.25;
    pa_.ihceq = 0;
    pa_.ppo = 8;
    pa_.rco = 1.2e6;
    pa_.rcp = 0.1;
    pa_.rdi = 100.0;
    pa_.red = 3.2;
    pa_.rfz = 1.0;
    pa_.rhe = 2500.0;
    pa_.rho = 1.0;
    pa_.rim = 4e2;
    pa_.rma = 60.0;
    pa_.rrw = 0;
    pa_.rst = 40.0;
    pa_.rvc = 200.0;
    pa_.rve = 0.28e6;
    pa_.stgain = 1e-8;
    pa_.stimfn[0] = '\0';
    pa_.tauhc = 0.5e-3;
    pa_.vhcmax = 1;
    pa_.vhcmin = 0;
    pa_.vihc=0;
    pa_.xhe = 0.0;
    pa_.xl = 3.5;
    pa_.xl1 = 0;
    pa_.xl2 = 0;
    pa_.yw = 0.1;
    pa_.zh = 0.1;

    pa_.apxacc = TRUE;
    pa_.basacc = TRUE;
    pa_.couplr = TRUE;
    pa_.midear = TRUE;

    pa_.aca = 0.01; pa_.acb = pa_.acc = pa_.acd = pa_.ace = 0;
    pa_.bwa = 0.50; pa_.bwb = pa_.bwc = pa_.bwd = pa_.bwe = 0;
    pa_.g1a = pa_.g1b = pa_.g1c = pa_.g1d = pa_.g1e = 0;
    pa_.g2a = pa_.g2b = pa_.g2c = pa_.g2d = pa_.g2e = 0;
    pa_.gba = pa_.gbb = pa_.gbc = pa_.gbd = pa_.gbe = 0;
    pa_.gpa = pa_.gpb = pa_.gpc = pa_.gpd = pa_.gpe = 0;
    pa_.k1a = pa_.k1b = pa_.k1c = pa_.k1d = pa_.k1e = 0;
    pa_.k2a = pa_.k2b = pa_.k2c = pa_.k2d = pa_.k2e = 0;
    pa_.k3a = pa_.k3b = pa_.k3c = pa_.k3d = pa_.k3e = 0;
    pa_.k4a = pa_.k4b = pa_.k4c = pa_.k4d = pa_.k4e = 0;
    pa_.k5a = pa_.k5b = pa_.k5c = pa_.k5d = pa_.k5e = 0;
    pa_.m1a = pa_.m1b = pa_.m1c = pa_.m1d = pa_.m1e = 0;
    pa_.m2a = pa_.m2b = pa_.m2c = pa_.m2d = pa_.m2e = 0;
    pa_.r1a = pa_.r1b = pa_.r1c = pa_.r1d = pa_.r1e = 0;
    pa_.r2a = pa_.r2b = pa_.r2c = pa_.r2d = pa_.r2e = 0;
    pa_.r3a = pa_.r3b = pa_.r3c = pa_.r3d = pa_.r3e = 0;
    pa_.r4a = pa_.r4b = pa_.r4c = pa_.r4d = pa_.r4e = 0;
    pa_.rfa = pa_.rfb = pa_.rfc = pa_.rfd = pa_.rfe = 0;
    pa_.t1a = pa_.t1b = pa_.t1c = pa_.t1d = pa_.t1e = 0;
    pa_.t2a = pa_.t2b = pa_.t2c = pa_.t2d = pa_.t2e = 0;

}

void
readpa(ifn)
char *ifn;
{
    char s[80];
    FILE *ifp;

    if (*ifn == '#') {
	pa_.ifrmt = 8;
	pa_.jfrmt = 3;
    } else if (*ifn == '+') {
	pa_.ifrmt = 0;
	pa_.jfrmt = 0;
    } else {
	if ((ifp = fopen(ifn, "r")) == NULL) {
	    sprintf(s, "can't open %s", ifn);
	    error(s);
	}
	fprintf(lfp, "reading %s\n", ifn);
	while (fgets(s, 80, ifp) != NULL)
	    parline(s);
	fclose(ifp);
    }
}

void
parline(s)
char *s;
{
    int i, j;
    char *e, *vs;
    double v;
    char *strchr();
    double atof();

    if (s[0] == '#')
	return;
    e = strchr(s, '=');
    if (e == NULL)
	return;
    vs = e + 1;
    while(isspace((int)*vs))		/* skip leading white space */
	vs++;
    for(i = 0; vs[i]; i++)		/* trim trailing white space */
	if (isspace((int)vs[i])) {
	    vs[i] = '\0';
	    break;
	}

    v = atof(vs);
    for(j = e - s; j > 0 && s[j - 1] == ' '; j--)
	continue;
    s[j] = '\0';
    for(i = 0; i < j && s[i] == ' '; i++)
	continue;
    if (i < j)
	chkpar(s + i, vs, v);
}

void
seto(float *pv, double v)
{
    pv[0] = v;
    pv[1] = 0;
    pv[2] = 0;
    pv[3] = 0;
}

void
setx(float *pv, double v)
{
    pv[0] = v;
    pv[1] = 0;
    pv[2] = 0;
    pv[3] = 0;
    pv[4] = 0;
}

void
chkpar(char *s, char *vs, double v)
{
   if (strcmp(s, "stimfn") == 0) strncpy(pa_.stimfn, vs, 40);

   else if (strcmp(s, "aca") == 0) pa_.aca = v;
   else if (strcmp(s, "acb") == 0) pa_.acb = v;
   else if (strcmp(s, "acc") == 0) pa_.acc = v;
   else if (strcmp(s, "acp") == 0) pa_.acp = v;
   else if (strcmp(s, "adi") == 0) pa_.adi = v;
   else if (strcmp(s, "aed") == 0) pa_.aed = v;
   else if (strcmp(s, "ama") == 0) pa_.ama = v;
   else if (strcmp(s, "ast") == 0) pa_.ast = v;
   else if (strcmp(s, "bl") == 0) pa_.bl  = v;
   else if (strcmp(s, "bwa") == 0) pa_.bwa = v;
   else if (strcmp(s, "bwb") == 0) pa_.bwb = v;
   else if (strcmp(s, "bwc") == 0) pa_.bwc = v;
   else if (strcmp(s, "bwd") == 0) pa_.bwd = v;
   else if (strcmp(s, "bwe") == 0) pa_.bwe = v;
   else if (strcmp(s, "d3max") == 0) pa_.d3max = v;
   else if (strcmp(s, "dhbmax") == 0) pa_.dhbmax = v;
   else if (strcmp(s, "dhbmin") == 0) pa_.dhbmin = v;
   else if (strcmp(s, "dihc") == 0) pa_.dihc = v;
   else if (strcmp(s, "dt") == 0) pa_.dt = v;
   else if (strcmp(s, "fpma") == 0) pa_.fpma = v;
   else if (strcmp(s, "fpmb") == 0) pa_.fpmb = v;
   else if (strcmp(s, "fpmc") == 0) pa_.fpmc = v;
   else if (strcmp(s, "freq") == 0) pa_.freq = v;
   else if (strcmp(s, "g1a") == 0) pa_.g1a = v;
   else if (strcmp(s, "g1b") == 0) pa_.g1b = v;
   else if (strcmp(s, "g1c") == 0) pa_.g1c = v;
   else if (strcmp(s, "g1d") == 0) pa_.g1d = v;
   else if (strcmp(s, "g1e") == 0) pa_.g1e = v;
   else if (strcmp(s, "g2a") == 0) pa_.g2a = v;
   else if (strcmp(s, "g2b") == 0) pa_.g2b = v;
   else if (strcmp(s, "g2c") == 0) pa_.g2c = v;
   else if (strcmp(s, "g2d") == 0) pa_.g2d = v;
   else if (strcmp(s, "g2e") == 0) pa_.g2e = v;
   else if (strcmp(s, "gam") == 0) pa_.gam = v;
   else if (strcmp(s, "gamuns") == 0) pa_.gamuns = v;
   else if (strcmp(s, "gba") == 0) pa_.gba = v;
   else if (strcmp(s, "gbb") == 0) pa_.gbb = v;
   else if (strcmp(s, "gbc") == 0) pa_.gbc = v;
   else if (strcmp(s, "gmel") == 0) pa_.gmel = v;
   else if (strcmp(s, "gpa") == 0) pa_.gpa = v;
   else if (strcmp(s, "gpb") == 0) pa_.gpb = v;
   else if (strcmp(s, "gpc") == 0) pa_.gpc = v;
   else if (strcmp(s, "gpd") == 0) pa_.gpd = v;
   else if (strcmp(s, "gpe") == 0) pa_.gpe = v;
   else if (strcmp(s, "icyc") == 0) pa_.icyc = v;
   else if (strcmp(s, "id1") == 0) pa_.id1 = v;
   else if (strcmp(s, "ifrmt") == 0) pa_.ifrmt = v;
   else if (strcmp(s, "ihceq") == 0) pa_.ihceq = v;
   else if (strcmp(s, "isv[0]") == 0) pa_.isv[0] = v;
   else if (strcmp(s, "isv[1]") == 0) pa_.isv[1] = v;
   else if (strcmp(s, "isv[2]") == 0) pa_.isv[2] = v;
   else if (strcmp(s, "isv[3]") == 0) pa_.isv[3] = v;
   else if (strcmp(s, "isv[4]") == 0) pa_.isv[4] = v;
   else if (strcmp(s, "isv[5]") == 0) pa_.isv[5] = v;
   else if (strcmp(s, "itsc") == 0) pa_.itsc = v;
   else if (strcmp(s, "itsf") == 0) pa_.itsf = v;
   else if (strcmp(s, "itsw") == 0) pa_.itsw = v;
   else if (strcmp(s, "jfrmt") == 0) pa_.jfrmt = v;
   else if (strcmp(s, "k1a") == 0) pa_.k1a = v;
   else if (strcmp(s, "k1b") == 0) pa_.k1b = v;
   else if (strcmp(s, "k1c") == 0) pa_.k1c = v;
   else if (strcmp(s, "k1d") == 0) pa_.k1d = v;
   else if (strcmp(s, "k1e") == 0) pa_.k1e = v;
   else if (strcmp(s, "k2a") == 0) pa_.k2a = v;
   else if (strcmp(s, "k2b") == 0) pa_.k2b = v;
   else if (strcmp(s, "k2c") == 0) pa_.k2c = v;
   else if (strcmp(s, "k2d") == 0) pa_.k2d = v;
   else if (strcmp(s, "k2e") == 0) pa_.k2e = v;
   else if (strcmp(s, "k3a") == 0) pa_.k3a = v;
   else if (strcmp(s, "k3b") == 0) pa_.k3b = v;
   else if (strcmp(s, "k3c") == 0) pa_.k3c = v;
   else if (strcmp(s, "k3d") == 0) pa_.k3d = v;
   else if (strcmp(s, "k3e") == 0) pa_.k3e = v;
   else if (strcmp(s, "k4a") == 0) pa_.k4a = v;
   else if (strcmp(s, "k4b") == 0) pa_.k4b = v;
   else if (strcmp(s, "k4c") == 0) pa_.k4c = v;
   else if (strcmp(s, "k4d") == 0) pa_.k4d = v;
   else if (strcmp(s, "k4e") == 0) pa_.k4e = v;
   else if (strcmp(s, "k5a") == 0) pa_.k5a = v;
   else if (strcmp(s, "k5b") == 0) pa_.k5b = v;
   else if (strcmp(s, "k5c") == 0) pa_.k5c = v;
   else if (strcmp(s, "k5d") == 0) pa_.k5d = v;
   else if (strcmp(s, "k5e") == 0) pa_.k5e = v;
   else if (strcmp(s, "kcp") == 0) pa_.kcp = v;
   else if (strcmp(s, "kdi") == 0) pa_.kdi = v;
   else if (strcmp(s, "ked") == 0) pa_.ked = v;
   else if (strcmp(s, "khe") == 0) pa_.khe = v;
   else if (strcmp(s, "kim") == 0) pa_.kim = v;
   else if (strcmp(s, "kma") == 0) pa_.kma = v;
   else if (strcmp(s, "krw") == 0) pa_.krw = v;
   else if (strcmp(s, "kst") == 0) pa_.kst = v;
   else if (strcmp(s, "m") == 0) pa_.m = v;
   else if (strcmp(s, "m1a") == 0) pa_.m1a = v;
   else if (strcmp(s, "m1b") == 0) pa_.m1b = v;
   else if (strcmp(s, "m1c") == 0) pa_.m1c = v;
   else if (strcmp(s, "m1d") == 0) pa_.m1d = v;
   else if (strcmp(s, "m1e") == 0) pa_.m1e = v;
   else if (strcmp(s, "m2a") == 0) pa_.m2a = v;
   else if (strcmp(s, "m2b") == 0) pa_.m2b = v;
   else if (strcmp(s, "m2c") == 0) pa_.m2c = v;
   else if (strcmp(s, "m2d") == 0) pa_.m2d = v;
   else if (strcmp(s, "m2e") == 0) pa_.m2e = v;
   else if (strcmp(s, "m3a") == 0) pa_.m3a = v;
   else if (strcmp(s, "m3b") == 0) pa_.m3b = v;
   else if (strcmp(s, "m3c") == 0) pa_.m3c = v;
   else if (strcmp(s, "m3d") == 0) pa_.m3d = v;
   else if (strcmp(s, "m3e") == 0) pa_.m3e = v;
   else if (strcmp(s, "mco") == 0) pa_.mco = v;
   else if (strcmp(s, "mcp") == 0) pa_.mcp = v;
   else if (strcmp(s, "mdi") == 0) pa_.mdi = v;
   else if (strcmp(s, "med") == 0) pa_.med = v;
   else if (strcmp(s, "mhe") == 0) pa_.mhe = v;
   else if (strcmp(s, "mma") == 0) pa_.mma = v;
   else if (strcmp(s, "mmeq") == 0) pa_.mmeq = v;
   else if (strcmp(s, "mst") == 0) pa_.mst = v;
   else if (strcmp(s, "n") == 0) pa_.n = v;
   else if (strcmp(s, "ncyc") == 0) pa_.ncyc = v;
   else if (strcmp(s, "ncdp") == 0) pa_.ncdp = v;
   else if (strcmp(s, "nimp") == 0) pa_.nimp = v;
   else if (strcmp(s, "ntsc") == 0) pa_.ntsc = v;
   else if (strcmp(s, "ntsf") == 0) pa_.ntsf = v;
   else if (strcmp(s, "ntsgat") == 0) pa_.ntsgat = v;
   else if (strcmp(s, "ntsw") == 0) pa_.ntsw = v;
   else if (strcmp(s, "ohccmp") == 0) pa_.ohccmp = v;
   else if (strcmp(s, "ohceq") == 0) pa_.ohceq = v;
   else if (strcmp(s, "ppo") == 0) pa_.ppo = v;
   else if (strcmp(s, "r1a") == 0) pa_.r1a = v;
   else if (strcmp(s, "r1b") == 0) pa_.r1b = v;
   else if (strcmp(s, "r1c") == 0) pa_.r1c = v;
   else if (strcmp(s, "r1d") == 0) pa_.r1d = v;
   else if (strcmp(s, "r1e") == 0) pa_.r1e = v;
   else if (strcmp(s, "r2a") == 0) pa_.r2a = v;
   else if (strcmp(s, "r2b") == 0) pa_.r2b = v;
   else if (strcmp(s, "r2c") == 0) pa_.r2c = v;
   else if (strcmp(s, "r2d") == 0) pa_.r2d = v;
   else if (strcmp(s, "r2e") == 0) pa_.r2e = v;
   else if (strcmp(s, "r3a") == 0) pa_.r3a = v;
   else if (strcmp(s, "r3b") == 0) pa_.r3b = v;
   else if (strcmp(s, "r3c") == 0) pa_.r3c = v;
   else if (strcmp(s, "r3d") == 0) pa_.r3d = v;
   else if (strcmp(s, "r3e") == 0) pa_.r3e = v;
   else if (strcmp(s, "r4a") == 0) pa_.r4a = v;
   else if (strcmp(s, "r4b") == 0) pa_.r4b = v;
   else if (strcmp(s, "r4c") == 0) pa_.r4c = v;
   else if (strcmp(s, "r4d") == 0) pa_.r4d = v;
   else if (strcmp(s, "r4e") == 0) pa_.r4e = v;
   else if (strcmp(s, "rco") == 0) pa_.rco = v;
   else if (strcmp(s, "rcp") == 0) pa_.rcp = v;
   else if (strcmp(s, "rdi") == 0) pa_.rdi = v;
   else if (strcmp(s, "red") == 0) pa_.red = v;
   else if (strcmp(s, "rfa") == 0) pa_.rfa = v;
   else if (strcmp(s, "rfb") == 0) pa_.rfb = v;
   else if (strcmp(s, "rfc") == 0) pa_.rfc = v;
   else if (strcmp(s, "rfd") == 0) pa_.rfd = v;
   else if (strcmp(s, "rfe") == 0) pa_.rfe = v;
   else if (strcmp(s, "rfz") == 0) pa_.rfz = v;
   else if (strcmp(s, "rhe") == 0) pa_.rhe = v;
   else if (strcmp(s, "rho") == 0) pa_.rho = v;
   else if (strcmp(s, "rim") == 0) pa_.rim = v;
   else if (strcmp(s, "rma") == 0) pa_.rma = v;
   else if (strcmp(s, "rrw") == 0) pa_.rrw = v;
   else if (strcmp(s, "rst") == 0) pa_.rst = v;
   else if (strcmp(s, "rvc") == 0) pa_.rvc = v;
   else if (strcmp(s, "rve") == 0) pa_.rve = v;
   else if (strcmp(s, "stgain") == 0) pa_.stgain = v;
   else if (strcmp(s, "sv0") == 0) pa_.isv[0] = v;
   else if (strcmp(s, "sv1") == 0) pa_.isv[1] = v;
   else if (strcmp(s, "sv2") == 0) pa_.isv[2] = v;
   else if (strcmp(s, "sv3") == 0) pa_.isv[3] = v;
   else if (strcmp(s, "sv4") == 0) pa_.isv[4] = v;
   else if (strcmp(s, "sv5") == 0) pa_.isv[5] = v;
   else if (strcmp(s, "t1a") == 0) pa_.t1a = v;
   else if (strcmp(s, "t1b") == 0) pa_.t1b = v;
   else if (strcmp(s, "t1c") == 0) pa_.t1c = v;
   else if (strcmp(s, "t1d") == 0) pa_.t1d = v;
   else if (strcmp(s, "t1e") == 0) pa_.t1e = v;
   else if (strcmp(s, "t2a") == 0) pa_.t2a = v;
   else if (strcmp(s, "t2b") == 0) pa_.t2b = v;
   else if (strcmp(s, "t2c") == 0) pa_.t2c = v;
   else if (strcmp(s, "t2d") == 0) pa_.t2d = v;
   else if (strcmp(s, "t2e") == 0) pa_.t2e = v;
   else if (strcmp(s, "tauhc") == 0) pa_.tauhc = v;
   else if (strcmp(s, "vhcmax") == 0) pa_.vhcmax = v;
   else if (strcmp(s, "vhcmin") == 0) pa_.vhcmin = v;
   else if (strcmp(s, "vihc") == 0) pa_.vihc = v;
   else if (strcmp(s, "xhe") == 0) pa_.xhe = v;
   else if (strcmp(s, "xl") == 0) pa_.xl = v;
   else if (strcmp(s, "yw") == 0) pa_.yw = v;
   else if (strcmp(s, "zh") == 0) pa_.zh = v;

   else if (strcmp(s, "apxacc") == 0) pa_.apxacc = v;
   else if (strcmp(s, "basacc") == 0) pa_.basacc = v;
   else if (strcmp(s, "couplr") == 0) pa_.couplr = v;
   else if (strcmp(s, "midear") == 0) pa_.midear = v;

   else if (strcmp(s, "xl1") == 0) pa_.xl1 = v;
   else if (strcmp(s, "xl2") == 0) pa_.xl2 = v;
   else if (strcmp(s, "f1") == 0) pa_.f1 = v;
   else if (strcmp(s, "f2") == 0) pa_.f2 = v;
   else if (strcmp(s, "f3") == 0) pa_.f1 = v;
   else if (strcmp(s, "f4") == 0) pa_.f2 = v;
   else if (strcmp(s, "l1") == 0) pa_.l1 = v;
   else if (strcmp(s, "l2") == 0) pa_.l2 = v;
   else if (strcmp(s, "l3") == 0) pa_.l1 = v;
   else if (strcmp(s, "l4") == 0) pa_.l2 = v;
   else if (strcmp(s, "k1r") == 0) pa_.k1r = v;
   else if (strcmp(s, "nrgn") == 0) pa_.nrgn = v;

   else if (strcmp(s, "rfo") == 0) seto(&pa_.rfa, v);
   else if (strcmp(s, "k1o") == 0) seto(&pa_.k1a, v);
   else if (strcmp(s, "r1o") == 0) seto(&pa_.r1a, v);
   else if (strcmp(s, "m1o") == 0) seto(&pa_.m1a, v);
   else if (strcmp(s, "k2o") == 0) seto(&pa_.k2a, v);
   else if (strcmp(s, "r2o") == 0) seto(&pa_.r2a, v);
   else if (strcmp(s, "m2o") == 0) seto(&pa_.m2a, v);
   else if (strcmp(s, "k3o") == 0) seto(&pa_.k3a, v);
   else if (strcmp(s, "r3o") == 0) seto(&pa_.r3a, v);
   else if (strcmp(s, "m3o") == 0) seto(&pa_.m3a, v);
   else if (strcmp(s, "k4o") == 0) seto(&pa_.k4a, v);
   else if (strcmp(s, "k5o") == 0) seto(&pa_.k5a, v);
   else if (strcmp(s, "r4o") == 0) seto(&pa_.r4a, v);
   else if (strcmp(s, "aco") == 0) seto(&pa_.aca, v);
   else if (strcmp(s, "bwo") == 0) seto(&pa_.bwa, v);
   else if (strcmp(s, "gpo") == 0) seto(&pa_.gpa, v);
   else if (strcmp(s, "g1o") == 0) seto(&pa_.g1a, v);
   else if (strcmp(s, "g2o") == 0) seto(&pa_.g2a, v);
   else if (strcmp(s, "t1o") == 0) seto(&pa_.t1a, v);
   else if (strcmp(s, "t2o") == 0) seto(&pa_.t2a, v);

   else if (strcmp(s, "rf") == 0) setx(&pa_.rfa, v);
   else if (strcmp(s, "k1") == 0) setx(&pa_.k1a, v);
   else if (strcmp(s, "r1") == 0) setx(&pa_.r1a, v);
   else if (strcmp(s, "m1") == 0) setx(&pa_.m1a, v);
   else if (strcmp(s, "k2") == 0) setx(&pa_.k2a, v);
   else if (strcmp(s, "r2") == 0) setx(&pa_.r2a, v);
   else if (strcmp(s, "m2") == 0) setx(&pa_.m2a, v);
   else if (strcmp(s, "k3") == 0) setx(&pa_.k3a, v);
   else if (strcmp(s, "r3") == 0) setx(&pa_.r3a, v);
   else if (strcmp(s, "m3") == 0) setx(&pa_.m3a, v);
   else if (strcmp(s, "k4") == 0) setx(&pa_.k4a, v);
   else if (strcmp(s, "k5") == 0) setx(&pa_.k5a, v);
   else if (strcmp(s, "r4") == 0) setx(&pa_.r4a, v);
   else if (strcmp(s, "ac") == 0) setx(&pa_.aca, v);
   else if (strcmp(s, "bw") == 0) setx(&pa_.bwa, v);
   else if (strcmp(s, "gp") == 0) setx(&pa_.gpa, v);
   else if (strcmp(s, "g1") == 0) setx(&pa_.g1a, v);
   else if (strcmp(s, "g2") == 0) setx(&pa_.g2a, v);
   else if (strcmp(s, "t1") == 0) setx(&pa_.t1a, v);
   else if (strcmp(s, "t2") == 0) setx(&pa_.t2a, v);

   else if (strcmp(s, "mfo") == 0)	/*** ignore ***/;
   else if (strcmp(s, "d3min") == 0)	/*** ignore ***/;

   else fprintf(lfp, "unknown parameter '%s'\n", s);
}

void
prnabc(FILE *fp, char *pn, float *pv)
{
    if (pv[0] == 0 || (pv[1] == 0 && pv[2] == 0 && pv[4] == 0)) {
        fprintf(fp, "  %s                  = %13.4g\n", pn, pv[0]);
    } else if (pv[1] == 0 && pv[2] == 0) {
        fprintf(fp, "  %so,%se             = ", pn, pn);
        fprintf(fp, "%13.4g%13.4g\n", pv[0], pv[4]);
    } else {
        fprintf(fp, "  %sa,%sb,%sc         = ", pn, pn, pn);
        fprintf(fp, "%13.4g%13.4g%13.4g\n", pv[0], pv[1], pv[2]);
    }
}

/* wrpar - write parameters to data output */

void
wrpar(FILE *fp)
{
    fprintf(fp, "\n");
    fprintf(fp, " Time-domain Model of Cochlear Mechanics  ");
    fprintf(fp, " version %.2f\n", pa_.vernum + 0.01 * pa_.revnum);
    fprintf(fp, "\n");
    fprintf(fp, "  m,n,mmeq,id1,ncdp   = %6d%6d%6d%6d%6d\n",
	pa_.m, pa_.n, pa_.mmeq, pa_.id1, pa_.ncdp);
    fprintf(fp, "  xl,yw,zh,rho        = %13.2f%13.2f%13.2f%13.2f\n",
	pa_.xl, pa_.yw, pa_.zh,pa_.rho);
    prnabc(fp, "rf", &pa_.rfa);
    prnabc(fp, "ac", &pa_.aca);
    prnabc(fp, "bw", &pa_.bwa);
    prnabc(fp, "gp", &pa_.gpa);
    prnabc(fp, "k1", &pa_.k1a);
    prnabc(fp, "r1", &pa_.r1a);
    prnabc(fp, "m1", &pa_.m1a);
    prnabc(fp, "k2", &pa_.k2a);
    prnabc(fp, "r2", &pa_.r2a);
    prnabc(fp, "m2", &pa_.m2a);
    prnabc(fp, "k3", &pa_.k3a);
    prnabc(fp, "r3", &pa_.r3a);
    prnabc(fp, "m3", &pa_.m3a);
    prnabc(fp, "k4", &pa_.k4a);
    prnabc(fp, "r4", &pa_.r4a);
    prnabc(fp, "g1", &pa_.g1a);
    prnabc(fp, "g2", &pa_.g2a);
    prnabc(fp, "k5", &pa_.k5a);
    fprintf(fp, "  kst,rst,mst,ast     = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.kst, pa_.rst, pa_.mst, pa_.ast);
    fprintf(fp, "  kma,rma,mma,ama     = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.kma, pa_.rma, pa_.mma, pa_.ama);
    fprintf(fp, "  kim,rim,gmel        = %13.4g%13.4g%13.4g\n",
	pa_.kim, pa_.rim, pa_.gmel);
    fprintf(fp, "  ked,red,med,aed     = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.ked, pa_.red, pa_.med, pa_.aed);
    fprintf(fp, "  kcp,rcp,mcp,acp     = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.kcp, pa_.rcp, pa_.mcp, pa_.acp);
    fprintf(fp, "  kdi,rdi,mdi,adi     = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.kdi, pa_.rdi, pa_.mdi, pa_.adi);
    fprintf(fp, "  bl,rvc,rfz          = %13.4g%13.4g%13.4g\n",
	pa_.bl, pa_.rvc, pa_.rfz);
    fprintf(fp, "  vhcmin,vhcmax,tauhc = %13.4g%13.4g%13.4g\n",
	pa_.vhcmin, pa_.vhcmax, pa_.tauhc);
    fprintf(fp, "  d3max,ohccmp,k1r    = %13.4g%13.4g%13.4g\n",
	pa_.d3max, pa_.ohccmp, pa_.k1r);
    fprintf(fp, "  dhbmin,dhbmax,nrgn  = %13.4g%13.4g%13.4g\n",
	pa_.dhbmin, pa_.dhbmax, pa_.nrgn);
    fprintf(fp, "  khe,rhe,mhe,xhe     = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.khe, pa_.rhe, pa_.mhe, pa_.xhe);
    fprintf(fp, "  rco,mco,rve,krw = %13.4g%13.4g%13.4g%13.4g\n",
	pa_.rco, pa_.mco, pa_.rve, pa_.krw);
    fprintf(fp, "  dt,stgain,ntsgat    = %13.4g%13.4g%13.1f\n",
	pa_.dt, pa_.stgain, pa_.ntsgat);
    fprintf(fp, "  stimfn              = %s\n",
	pa_.stimfn);
    fprintf(fp, "  ohceq,ihceq,nimp    = %4d%4d%4d\n",
	pa_.ohceq, pa_.ihceq, pa_.nimp);
    fprintf(fp, "  cpl,mde,bas,apx     = %4d%4d%4d%4d\n",
	pa_.couplr, pa_.midear, pa_.basacc, pa_.apxacc);
    fprintf(fp, "  ireor,ifrmt,jfrmt   = %4d%4d%4d\n",
	pa_.ireor, pa_.ifrmt, pa_.jfrmt);
    fprintf(fp, "  isv                 = %4d%4d%4d%4d%4d\n",
	pa_.isv[0], pa_.isv[1], pa_.isv[2], pa_.isv[3], pa_.isv[4]);
    fprintf(fp, "  itsc,icyc,itsw,itsf = %8ld%8ld%8ld%8ld\n",
	pa_.itsc, pa_.icyc, pa_.itsw, pa_.itsf);
    fprintf(fp, "  ntsc,ncyc,ntsw,ntsf = %8ld%8ld%8ld%8ld\n",
	pa_.ntsc, pa_.ncyc, pa_.ntsw, pa_.ntsf);
    fprintf(fp, "  f1,f2,f3,f4         = %8.0f%8.0f%8.0f%8.0f\n",
	pa_.f1, pa_.f2, pa_.f3, pa_.f4);
    fprintf(fp, "  l1,l2,l3,l4         = %8.0f%8.0f%8.0f%8.0f\n",
	pa_.l1, pa_.l2, pa_.l3, pa_.l4);
    fprintf(fp, "  xl1,xl2             = %8.2f%8.2f\n",
	pa_.xl1, pa_.xl2);
    fprintf(fp, "  fpma,fpmb,fpmc      = %8.4f%8.4f%8.4f\n",
	pa_.fpma, pa_.fpmb, pa_.fpmc);
}
