/* tdmsub - vector subroutines for tdm */

/* vmov - vector move */

void
vmov(float *v1, float *v2, int n)
{
    while (n-- > 0)
	*v2++ = *v1++;
}

/* vzero - vector zero */

void
vzero(float *v, int n)
{
    while (n-- > 0)
	*v++ = 0.0;
}

/* vsub - vector subtract */

void
vsub(float *v1, float *v2, float *v3, int n)
{
    while (n-- > 0)
	*v3++ = *v1++ - *v2++;
}

/* sve - sum of vector elements */

void
sve(float *v, float *s, int n)
{
    *s = 0;
    while (n-- > 0)
	*s += *v++;
}

/* vmxmg - maximum magnitude of vector elements */

void
vmxmg(float *v, float *s, int *k, int n)
{
    int i;

    *s = *k = 0;
    for (i = 0; i < n; i++) {
	if (*s < v[i]) {	
	    *s = v[i];
	    *k = i;
	} else if (*s < -v[i]) {
	    *s = -v[i];
	    *k = i;
	}
    }
}

/* vppk - positive peak of vector elements */

void
vppk(float *v, float *s, int *k, int n)
{
    int i;

    *s = *k = 0;
    for (i = 0; i < n; i++) {
	if (*s < v[i]) {	
	    *s = v[i];
	    *k = i;
	}
    }
}

/* vcen - centroid of vector elements */

void
vcen(float *v, float *c, int n)
{
    int i;
    double vsq, sum1, sum2;

    sum1 = sum2 = 0;
    for (i = 0; i < n; i++) {
	vsq = v[i] * v[i];
	sum1 += vsq;
	sum2 += vsq * i;
    }
    *c = (sum1 > 0 && n > 1) ? (sum2 / sum1) / (n - 1) : 0;
}
